# -*- coding: utf-8 -*-
'''
Return salt data via pushover (http://www.pushover.net)

.. versionadded:: Boron

The following fields can be set in the minion conf file::

    pushover.user (required)
    pushover.token (required)
    pushover.title (optional)
    pushover.device (optional)
    pushover.priority (optional)
    pushover.expire (optional)
    pushover.retry (optional)
    pushover.profile (optional)

Alternative configuration values can be used by prefacing the configuration.
Any values not found in the alternative configuration will be pulled from
the default location::

    alternative.pushover.user
    alternative.pushover.token
    alternative.pushover.title
    alternative.pushover.device
    alternative.pushover.priority
    alternative.pushover.expire
    alternative.pushover.retry

PushOver settings may also be configured as::

    pushover:
        user: xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
        token: xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
        title: Salt Returner
        device: phone
        priority: -1
        expire: 3600
        retry: 5

    alternative.pushover:
        user: xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
        token: xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
        title: Salt Returner
        device: phone
        priority: 1
        expire: 4800
        retry: 2

    pushover_profile:
        token: xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

    pushover:
        user: xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
        profile: pushover_profile

    alternative.pushover:
        user: xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
        profile: pushover_profile

  To use the PushOver returner, append '--return pushover' to the salt command. ex:

  .. code-block:: bash

    salt '*' test.ping --return pushover

  To use the alternative configuration, append '--return_config alternative' to the salt command. ex:

    salt '*' test.ping --return pushover --return_config alternative
'''
from __future__ import absolute_import

# Import Python libs
import pprint
import logging
import urllib

# Import 3rd-party libs
# pylint: disable=import-error,no-name-in-module,redefined-builtin
from salt.ext.six.moves.urllib.parse import urljoin as _urljoin  # pylint: disable=import-error,no-name-in-module
import salt.ext.six.moves.http_client
# pylint: enable=import-error,no-name-in-module,redefined-builtin

# Import Salt Libs
import salt.returners
import salt.utils.http
from salt.exceptions import SaltInvocationError

log = logging.getLogger(__name__)

__virtualname__ = 'pushover'


def _get_options(ret=None):
    '''
    Get the pushover options from salt.
    '''

    defaults = {'priority': '0'}

    attrs = {'pushover_profile': 'profile',
             'user': 'user',
             'device': 'device',
             'token': 'token',
             'priority': 'priority',
             'title': 'title',
             'api_version': 'api_version',
             'expire': 'expire',
             'retry': 'retry',
             'sound': 'sound',
             }

    profile_attr = 'pushover_profile'

    profile_attrs = {'user': 'user',
                     'device': 'device',
                     'token': 'token',
                     'priority': 'priority',
                     'title': 'title',
                     'api_version': 'api_version',
                     'expire': 'expire',
                     'retry': 'retry',
                     'sound': 'sound',
                     }

    _options = salt.returners.get_returner_options(__virtualname__,
                                                   ret,
                                                   attrs,
                                                   profile_attr=profile_attr,
                                                   profile_attrs=profile_attrs,
                                                   __salt__=__salt__,
                                                   __opts__=__opts__,
                                                   defaults=defaults)
    return _options


def __virtual__():
    '''
    Return virtual name of the module.

    :return: The virtual name of the module.
    '''
    return __virtualname__


def _query(function,
           token=None,
           api_version='1',
           method='POST',
           header_dict=None,
           data=None,
           query_params=None):
    '''
    PushOver object method function to construct and execute on the API URL.

    :param token:       The PushOver api key.
    :param api_version: The PushOver API version to use, defaults to version 1.
    :param function:    The PushOver api function to perform.
    :param method:      The HTTP method, e.g. GET or POST.
    :param data:        The data to be sent for POST method.
    :return:            The json response from the API call or False.
    '''
    if query_params is None:
        query_params = {}

    ret = {'message': '',
           'res': True}

    pushover_functions = {
        'message': {
            'request': 'messages.json',
            'response': 'status',
        },
        'validate_user': {
            'request': 'users/validate.json',
            'response': 'status',
        },
        'validate_sound': {
            'request': 'sounds.json',
            'response': 'status',
        },
    }

    api_url = 'https://api.pushover.net'
    base_url = _urljoin(api_url, api_version + '/')
    path = pushover_functions.get(function).get('request')
    url = _urljoin(base_url, path, False)

    if not query_params:
        query_params = {}

    decode = True
    if method == 'DELETE':
        decode = False

    result = salt.utils.http.query(
        url,
        method,
        params=query_params,
        data=data,
        header_dict=header_dict,
        decode=decode,
        decode_type='json',
        text=True,
        status=True,
        cookies=True,
        persist_session=True,
        opts=__opts__,
    )

    if result.get('status', None) == salt.ext.six.moves.http_client.OK:
        response = pushover_functions.get(function).get('response')
        if response in result and result[response] == 0:
            ret['res'] = False
        ret['message'] = result
        return ret
    else:
        try:
            if 'response' in result and result[response] == 0:
                ret['res'] = False
            ret['message'] = result
            return ret
        except ValueError:
            ret['res'] = False
            ret['message'] = result
            return ret


def _validate_sound(sound,
                    token):
    '''
    Validate that the sound sent to Pushover exists.
    :param sound:       The sound that we want to verify
    :param token:       The PushOver token.
    '''
    res = {
            'message': 'Sound is invalid',
            'result': False
           }
    parameters = dict()
    parameters['token'] = token

    response = _query(function='validate_sound',
                      method='GET',
                      query_params=parameters)

    if response['res']:
        log.debug('response {0}'.format(response))
        if 'message' in response:
            _message = response.get('message', '')
            if 'status' in _message:
                if _message.get('dict', {}).get('status', '') == 1:
                    sounds = _message.get('dict', {}).get('sounds', '')
                    if sound in sounds:
                        res['result'] = True
                        res['message'] = 'Sound is valid.'
                else:
                    res['message'] = ''.join(_message.get('dict', {}).get('errors'))
    return res


def _validate_user(user,
                   device,
                   token):
    '''
    Validate that a Pushover user or group exists.
    :param user:        The user or group name, either will work.
    :param device:      The device for the user.
    :param token:       The PushOver token.
    '''
    ret = {
            'message': 'User key is invalid',
            'res': False
           }

    parameters = dict()
    parameters['user'] = user
    parameters['token'] = token
    parameters['device'] = device

    response = _query(function='validate_user',
                      method='POST',
                      header_dict={'Content-Type': 'application/x-www-form-urlencoded'},
                      data=urllib.urlencode(parameters))

    if response['res']:
        if 'message' in response:
            _message = response.get('message', '')
            if 'status' in _message:
                if _message.get('dict', {}).get('status', None) == 1:
                    ret['res'] = True
                    ret['message'] = 'User key is valid.'
                else:
                    ret['res'] = False
                    ret['message'] = ''.join(_message.get('dict', {}).get('errors'))
    return ret


def _post_message(user,
                  device,
                  message,
                  title,
                  priority,
                  expire,
                  retry,
                  sound,
                  api_version=1,
                  token=None):
    '''
    Send a message to a Pushover user or group.
    :param user:        The user or group to send to, must be key of user or group not email address.
    :param message:     The message to send to the PushOver user or group.
    :param title:       Specify who the message is from.
    :param priority     The priority of the message, defaults to 0.
    :param api_version: The PushOver API version, if not specified in the configuration.
    :param notify:      Whether to notify the room, default: False.
    :param token:       The PushOver token, if not specified in the configuration.
    :return:            Boolean if message was sent successfully.
    '''

    user_validate = _validate_user(user, device, token)
    if not user_validate['res']:
        return user_validate

    parameters = dict()
    parameters['user'] = user
    parameters['device'] = device
    parameters['token'] = token
    parameters['title'] = title
    parameters['priority'] = priority
    parameters['expire'] = expire
    parameters['retry'] = retry
    parameters['message'] = message

    if sound:
        if _validate_sound(sound, token)['result']:
            parameters['sound'] = sound

    result = _query(function='message',
                    method='POST',
                    header_dict={'Content-Type': 'application/x-www-form-urlencoded'},
                    data=urllib.urlencode(parameters))

    return result


def returner(ret):
    '''
    Send an PushOver message with the data
    '''

    _options = _get_options(ret)

    user = _options.get('user')
    device = _options.get('device')
    token = _options.get('token')
    title = _options.get('title')
    priority = _options.get('priority')
    expire = _options.get('expire')
    retry = _options.get('retry')
    sound = _options.get('sound')

    if not token:
        raise SaltInvocationError('Pushover token is unavailable.')

    if not user:
        raise SaltInvocationError('Pushover user key is unavailable.')

    if priority and priority == 2:
        if not expire and not retry:
            raise SaltInvocationError('Priority 2 requires pushover.expire and pushover.retry options.')

    message = ('id: {0}\r\n'
               'function: {1}\r\n'
               'function args: {2}\r\n'
               'jid: {3}\r\n'
               'return: {4}\r\n').format(
                    ret.get('id'),
                    ret.get('fun'),
                    ret.get('fun_args'),
                    ret.get('jid'),
                    pprint.pformat(ret.get('return')))

    result = _post_message(user=user,
                           device=device,
                           message=message,
                           title=title,
                           priority=priority,
                           expire=expire,
                           retry=retry,
                           sound=sound,
                           token=token)

    if not result['res']:
        log.info('Error: {0}'.format(result['message']))
    return



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import
import sys
import types

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    Mock,
    NO_MOCK,
    NO_MOCK_REASON
)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# wmi and pythoncom modules are platform specific...
wmi = types.ModuleType('wmi')
sys.modules['wmi'] = wmi

pythoncom = types.ModuleType('pythoncom')
sys.modules['pythoncom'] = pythoncom

if NO_MOCK is False:
    WMI = Mock()
    wmi.WMI = Mock(return_value=WMI)
    pythoncom.CoInitialize = Mock()
    pythoncom.CoUninitialize = Mock()

# Import Salt Libs
from salt.modules import win_dns_client

# Globals
win_dns_client.__salt__ = {}
win_dns_client.__context__ = {}


class Mockwmi(object):
    '''
    Mock wmi class
    '''
    NetConnectionID = 'Local Area Connection'
    Index = 0
    DNSServerSearchOrder = ['10.1.1.10']
    Description = 'Local Area Connection'
    DHCPEnabled = True

    def __init__(self):
        pass


class Mockwinapi(object):
    '''
    Mock winapi class
    '''
    def __init__(self):
        pass

    class winapi(object):
        '''
        Mock winapi class
        '''
        def __init__(self):
            pass

        @staticmethod
        def Com():
            '''
            Mock Com method
            '''
            return True


@skipIf(NO_MOCK, NO_MOCK_REASON)
class WinDnsClientTestCase(TestCase):
    '''
    Test cases for salt.modules.win_dns_client
    '''
    # 'get_dns_servers' function tests: 1

    @patch('salt.utils', Mockwinapi)
    def test_get_dns_servers(self):
        '''
        Test if it return a list of the configured DNS servers
        of the specified interface.
        '''
        with patch('salt.utils.winapi.Com', MagicMock()):
            with patch.object(WMI, 'Win32_NetworkAdapter',
                              return_value=[Mockwmi()]):
                with patch.object(WMI, 'Win32_NetworkAdapterConfiguration',
                                  return_value=[Mockwmi()]):
                    self.assertListEqual(win_dns_client.get_dns_servers
                                         ('Local Area Connection'),
                                         ['10.1.1.10'])

                    self.assertFalse(win_dns_client.get_dns_servers('Ethernet'))

    # 'rm_dns' function tests: 1

    def test_rm_dns(self):
        '''
        Test if it remove the DNS server from the network interface.
        '''
        with patch.dict(win_dns_client.__salt__,
                        {'cmd.retcode': MagicMock(return_value=0)}):
            self.assertTrue(win_dns_client.rm_dns('10.1.1.10'))

    # 'add_dns' function tests: 1

    def test_add_dns(self):
        '''
        Test if it add the DNS server to the network interface.
        '''
        with patch('salt.utils.winapi.Com', MagicMock()):
            with patch.object(WMI, 'Win32_NetworkAdapter',
                              return_value=[Mockwmi()]):
                with patch.object(WMI, 'Win32_NetworkAdapterConfiguration',
                                  return_value=[Mockwmi()]):
                    self.assertFalse(win_dns_client.add_dns('10.1.1.10',
                                                            'Ethernet'))

                    self.assertTrue(win_dns_client.add_dns
                                    ('10.1.1.10', 'Local Area Connection'))

        with patch.object(win_dns_client, 'get_dns_servers',
                          MagicMock(return_value=['10.1.1.10'])):
            with patch.dict(win_dns_client.__salt__,
                            {'cmd.retcode': MagicMock(return_value=0)}):
                self.assertTrue(win_dns_client.add_dns('10.1.1.0',
                                                       'Local Area Connection'))

    # 'dns_dhcp' function tests: 1

    def test_dns_dhcp(self):
        '''
        Test if it configure the interface to get its
        DNS servers from the DHCP server
        '''
        with patch.dict(win_dns_client.__salt__,
                        {'cmd.retcode': MagicMock(return_value=0)}):
            self.assertTrue(win_dns_client.dns_dhcp())

    # 'get_dns_config' function tests: 1

    def test_get_dns_config(self):
        '''
        Test if it get the type of DNS configuration (dhcp / static)
        '''
        with patch('salt.utils.winapi.Com', MagicMock()):
            with patch.object(WMI, 'Win32_NetworkAdapter',
                              return_value=[Mockwmi()]):
                with patch.object(WMI, 'Win32_NetworkAdapterConfiguration',
                                  return_value=[Mockwmi()]):
                    self.assertTrue(win_dns_client.get_dns_config())


if __name__ == '__main__':
    from integration import run_tests
    run_tests(WinDnsClientTestCase, needs_daemon=False)



# -*- coding: UTF-8 -*-
'''
Splay function calls across targeted minions
'''

# Import Python Libs
from __future__ import absolute_import
import time

# Import 3rd-party libs
import salt.ext.six as six

# Import Salt Libs
from salt.exceptions import CommandExecutionError

_DEFAULT_SPLAYTIME = 600
_DEFAULT_SIZE = 8192


def _get_hash(hashable, size):
    '''
    Jenkins One-At-A-Time Hash Function
    More Info: http://en.wikipedia.org/wiki/Jenkins_hash_function#one-at-a-time
    '''
    # Using bitmask to emulate rollover behavior of C unsigned 32 bit int
    bitmask = 0xffffffff
    h = 0

    for i in bytearray(hashable):
        h = (h + i) & bitmask
        h = (h + (h << 10)) & bitmask
        h = (h ^ (h >> 6)) & bitmask

    h = (h + (h << 3)) & bitmask
    h = (h ^ (h >> 11)) & bitmask
    h = (h + (h << 15)) & bitmask

    return (h & (size - 1)) & bitmask


def _calc_splay(hashable, splaytime=_DEFAULT_SPLAYTIME, size=_DEFAULT_SIZE):
    hash_val = _get_hash(hashable, size)
    return int(splaytime * hash_val / float(size))


def splay(*args, **kwargs):
    '''
    Splay a salt function call execution time across minions over
    a number of seconds (default: 600)

    .. note::
        You *probably* want to use --async here and look up the job results later.
        If you're dead set on getting the output from the CLI command, then make
        sure to set the timeout (with the -t flag) to something greater than the
        splaytime (max splaytime + time to execute job).
        Otherwise, it's very likely that the cli will time out before the job returns.


    CLI Examples:

    .. code-block:: bash

        salt --async '*' splay.splay pkg.install cowsay version=3.03-8.el6

    .. code-block:: bash

      # With specified splaytime (5 minutes) and timeout with 10 second buffer
      salt -t 310 '*' splay.splay 300 pkg.version cowsay
    '''
    # Convert args tuple to a list so we can pop the splaytime and func out
    args = list(args)

    # If the first argument passed is an integer, set it as the splaytime
    try:
        splaytime = int(args[0])
        args.pop(0)
    except ValueError:
        splaytime = _DEFAULT_SPLAYTIME

    if splaytime <= 0:
        raise ValueError('splaytime must be a positive integer')

    func = args.pop(0)
    # Check if the func is valid before the sleep
    if func not in __salt__:
        raise CommandExecutionError('Unable to find module function {0}'.format(func))

    my_delay = _calc_splay(__grains__['id'], splaytime=splaytime)
    time.sleep(my_delay)
    # Get rid of the hidden kwargs that salt injects
    func_kwargs = dict((k, v) for k, v in six.iteritems(kwargs) if not k.startswith('__'))
    result = __salt__[func](*args, **func_kwargs)
    if not isinstance(result, dict):
        result = {'result': result}
    result['splaytime'] = str(my_delay)
    return result


def show(splaytime=_DEFAULT_SPLAYTIME):
    '''
    Show calculated splaytime for this minion
    Will use default value of 600 (seconds) if splaytime value not provided


    CLI Example:
        salt example-host splay.show
        salt example-host splay.show 60
    '''
    # Coerce splaytime to int (passed arg from CLI will be a str)
    if not isinstance(splaytime, int):
        splaytime = int(splaytime)

    return str(_calc_splay(__grains__['id'], splaytime=splaytime))



# -*- coding: utf-8 -*-
'''
Tests for the local_cache returner
'''
# Import Python libs
from __future__ import absolute_import
import logging
import os

# Import Salt Testing libs
from salttesting.helpers import ensure_in_syspath
ensure_in_syspath('../../')

# Import salt libs
import integration
from integration import TMP
import salt.utils.job
from salt.returners import local_cache

log = logging.getLogger(__name__)

# JOBS DIR and FILES
TMP_CACHE_DIR = os.path.join(TMP, 'master-minion-root', 'cache')
JOBS_DIR = os.path.join(TMP_CACHE_DIR, 'jobs')
JID_DIR = os.path.join(JOBS_DIR, 'b1', '4dac7bc743c928d7fac908823af1ab')
JID_FILE = os.path.join(JID_DIR, 'jid')
JID_MINION_DIR = os.path.join(JID_DIR, 'minion', 'return.p')
JOB_CACHE_DIR_FILES = [JID_FILE, JID_MINION_DIR]
KEEP_JOBS = 0.0000000010
EMPTY_JID_DIR = []

local_cache.__opts__ = {'cachedir': TMP_CACHE_DIR,
                        'keep_jobs': KEEP_JOBS}


class Local_CacheTest(integration.ShellCase):
    '''
    Test the local cache returner
    '''
    def _check_dir_files(self, msg, contents, status='None'):
        '''
        helper method to ensure files or dirs
        are either present or removed
        '''
        for content in contents:
            log.debug('CONTENT {0}'.format(content))
            if status == 'present':
                check_job_dir = os.path.exists(content)
            elif status == 'removed':
                if os.path.exists(content):
                    check_job_dir = False
                else:
                    check_job_dir = True
            self.assertTrue(check_job_dir,
                            msg=msg + content)

    def _add_job(self):
        '''
        helper method to add job.
        '''
        # add the job.
        opts = {}
        opts.update(self.get_config('master'))
        load = {'fun_args': [], 'jid': '20160603132323715452',
                'return': True, 'retcode': 0, 'success': True,
                'cmd': '_return', 'fun': 'test.ping', 'id': 'minion'}

        add_job = salt.utils.job.store_job(opts, load)
        self.assertEqual(add_job, None)
        self._check_dir_files('Dir/file does not exist: ',
                              JOB_CACHE_DIR_FILES,
                              status='present')

    def test_clean_old_jobs(self):
        '''
        test to ensure jobs are removed from job cache
        '''
        self._add_job()

        # remove job
        self.assertEqual(local_cache.clean_old_jobs(), None)

        self._check_dir_files('job cache was not removed: ',
                              JOB_CACHE_DIR_FILES,
                              status='removed')

    def test_not_clean_new_jobs(self):
        '''
        test to ensure jobs are not removed when
        jobs dir is new
        '''
        self._add_job()

        local_cache.__opts__['keep_jobs'] = 24
        self.assertEqual(local_cache.clean_old_jobs(), None)

        self._check_dir_files('job cache was removed: ',
                              JOB_CACHE_DIR_FILES,
                              status='present')

        # need to set back to initial KEEP_JOBS
        local_cache.__opts__['keep_jobs'] = KEEP_JOBS

    def test_empty_jid_dir(self):
        '''
        test to ensure removal of empty jid dir
        '''
        # add empty jid dir
        new_jid_dir = os.path.join(JOBS_DIR, 'z0')
        EMPTY_JID_DIR.append(new_jid_dir)
        os.makedirs(new_jid_dir)

        # check dir exists
        self._check_dir_files('new_jid_dir was not created',
                              EMPTY_JID_DIR,
                              status='present')

        # remove job
        self.assertEqual(local_cache.clean_old_jobs(), None)

        # check jid dir is removed
        self._check_dir_files('new_jid_dir was not removed',
                              EMPTY_JID_DIR,
                              status='removed')


if __name__ == '__main__':
    from integration import run_tests
    run_tests(Local_CacheTest)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)
from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import etcd_mod
from salt.utils import etcd_util

# Globals
etcd_mod.__opts__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class EtcdModTestCase(TestCase):
    '''
    Test cases for salt.modules.etcd_mod
    '''
    # 'get_' function tests: 1

    def test_get(self):
        '''
        Test if it get a value from etcd, by direct path
        '''
        class MockEtcd(object):
            """
            Mock of etcd
            """
            value = 'salt'

            def __init__(self):
                self.key = None

            def get(self, key):
                """
                Mock of get method
                """
                self.key = key
                if key == '':
                    raise KeyError
                elif key == 'err':
                    raise ValueError
                return MockEtcd

        with patch.object(etcd_util, 'get_conn',
                          MagicMock(return_value=MockEtcd())):
            self.assertEqual(etcd_mod.get_('salt'), 'salt')

            with patch.object(etcd_util, 'tree', MagicMock(return_value={})):
                self.assertDictEqual(etcd_mod.get_('salt', recurse=True), {})

            self.assertEqual(etcd_mod.get_(''), '')

            self.assertRaises(ValueError, etcd_mod.get_, 'err')

    # 'set_' function tests: 1

    def test_set(self):
        '''
        Test if it set a value in etcd, by direct path
        '''
        class MockEtcd(object):
            """
            Mock of etcd
            """
            value = 'salt'

            def __init__(self):
                self.key = None
                self.value = None

            def write(self, key, value):
                """
                Mock of write method
                """
                self.key = key
                self.value = value
                if key == '':
                    raise KeyError
                elif key == 'err':
                    raise ValueError
                return MockEtcd

        with patch.object(etcd_util, 'get_conn',
                          MagicMock(return_value=MockEtcd())):
            self.assertEqual(etcd_mod.set_('salt', 'stack'), 'salt')

            self.assertEqual(etcd_mod.set_('', 'stack'), '')

            self.assertRaises(ValueError, etcd_mod.set_, 'err', 'stack')

    # 'ls_' function tests: 1

    def test_ls(self):
        '''
        Test if it return all keys and dirs inside a specific path
        '''
        class MockEtcd(object):
            """
            Mock of etcd
            """
            children = []

            def __init__(self):
                self.path = None

            def get(self, path):
                """
                Mock of get method
                """
                self.path = path
                if path == '':
                    raise KeyError
                elif path == 'err':
                    raise ValueError
                return MockEtcd

        with patch.object(etcd_util, 'get_conn',
                          MagicMock(return_value=MockEtcd())):
            self.assertDictEqual(etcd_mod.ls_(), {'/': {}})

            self.assertDictEqual(etcd_mod.ls_(''), {})

            self.assertRaises(ValueError, etcd_mod.ls_, 'err')

    # 'rm_' function tests: 1

    def test_rm(self):
        '''
        Test if it delete a key from etcd
        '''
        class MockEtcd(object):
            """
            Mock of etcd
            """
            children = []

            def __init__(self):
                self.key = None
                self.recursive = None

            def delete(self, key, recursive):
                """
                Mock of delete method
                """
                self.key = key
                self.recursive = recursive
                if key == '':
                    raise KeyError
                elif key == 'err':
                    raise ValueError
                if recursive:
                    return True
                else:
                    return False
                return MockEtcd

        with patch.object(etcd_util, 'get_conn',
                          MagicMock(return_value=MockEtcd())):
            self.assertFalse(etcd_mod.rm_('salt'))

            self.assertTrue(etcd_mod.rm_('salt', recurse=True))

            self.assertFalse(etcd_mod.rm_(''))

            self.assertRaises(ValueError, etcd_mod.rm_, 'err')

    # 'tree' function tests: 1

    def test_tree(self):
        '''
        Test if it recurse through etcd and return all values
        '''
        class MockEtcd(object):
            """
            Mock of etcd
            """
            children = []

        with patch.object(etcd_util, 'get_conn',
                          MagicMock(return_value=MockEtcd())):
            with patch.object(etcd_util, 'tree', MagicMock(return_value={})):
                self.assertDictEqual(etcd_mod.tree(), {})

            with patch.object(etcd_util, 'tree',
                              MagicMock(side_effect=KeyError)):
                self.assertDictEqual(etcd_mod.tree(), {})

            with patch.object(etcd_util, 'tree',
                              MagicMock(side_effect=Exception)):
                self.assertRaises(Exception, etcd_mod.tree)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(EtcdModTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rahul Handay <rahulha@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import
import sys

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import network

# Globals
network.__salt__ = {}
network.__grains__ = {}
network.__opts__ = {}


class MockNetwork(object):
    '''
        Mock network class
    '''
    def __init__(self):
        pass

    @staticmethod
    def interfaces():
        '''
            Mock interface method
        '''
        return {'salt': {'up': 1}}


class MockGrains(object):
    '''
        Mock Grains class
    '''
    def __init__(self):
        pass

    @staticmethod
    def grains(lis, bol):
        '''
            Mock grains method
        '''
        return {'A': 'B'}


@skipIf(sys.version_info < (2, 7), 'This needs to be refactored to work with Python 2.6')
@skipIf(NO_MOCK, NO_MOCK_REASON)
class NetworkTestCase(TestCase):
    '''
        Validate the network state
    '''
    @patch('salt.states.network.salt.utils.network', MockNetwork())
    @patch('salt.states.network.salt.loader', MockGrains())
    def test_managed(self):
        '''
            Test to ensure that the named interface is configured properly
        '''
        ret = {'name': 'salt',
               'changes': {},
               'result': False,
               'comment': ''}

        change = {'interface': '--- \n+++ \n@@ -1 +1 @@\n-A\n+B',
                  'status': 'Interface salt restart to validate'}

        mock = MagicMock(side_effect=[AttributeError, 'A', 'A', 'A', 'A', 'A'])
        with patch.dict(network.__salt__, {"ip.get_interface": mock}):
            self.assertDictEqual(network.managed('salt',
                                                 'stack', test='a'), ret)

            mock = MagicMock(return_value='B')
            with patch.dict(network.__salt__, {"ip.build_interface": mock}):
                mock = MagicMock(side_effect=AttributeError)
                with patch.dict(network.__salt__, {"ip.get_bond": mock}):
                    self.assertDictEqual(network.managed('salt',
                                                         'bond',
                                                         test='a'), ret)

                ret.update({'comment': 'Interface salt is set to be'
                            ' updated:\n--- \n+++ \n@@ -1 +1 @@\n-A\n+B',
                            'result': None})
                self.assertDictEqual(network.managed('salt', 'stack',
                                                     test='a'), ret)

                mock = MagicMock(return_value=True)
                with patch.dict(network.__salt__, {"ip.down": mock}):
                    with patch.dict(network.__salt__, {"ip.up": mock}):
                        ret.update({'comment': 'Interface salt updated.',
                                    'result': True,
                                    'changes': change})
                        self.assertDictEqual(network.managed('salt', 'stack'),
                                             ret)

                        with patch.dict(network.__grains__, {"A": True}):
                            with patch.dict(network.__salt__,
                                            {"saltutil.refresh_modules": mock}
                                            ):
                                ret.update({'result': True,
                                            'changes': {'interface': '--- \n+'
                                                        '++ \n@@ -1 +1 @@\n-A'
                                                        '\n+B',
                                                        'status': 'Interface'
                                                        ' salt down'}})
                                self.assertDictEqual(network.managed('salt',
                                                                     'stack',
                                                                     False),
                                                     ret)

                ret.update({'changes': {'interface':
                                        '--- \n+++ \n@@ -1 +1 @@\n-A\n+B'},
                            'result': False,
                            'comment': "'ip.down'"})
                self.assertDictEqual(network.managed('salt', 'stack'), ret)

    def test_routes(self):
        '''
            Test to manage network interface static routes.
        '''
        ret = {'name': 'salt',
               'changes': {},
               'result': False,
               'comment': ''}

        mock = MagicMock(side_effect=[AttributeError, False, False, "True",
                                      False, False])
        with patch.dict(network.__salt__, {"ip.get_routes": mock}):
            self.assertDictEqual(network.routes('salt'), ret)

            mock = MagicMock(side_effect=[False, True, '', True, True])
            with patch.dict(network.__salt__, {"ip.build_routes": mock}):
                ret.update({'result': True,
                            'comment': 'Interface salt routes are up to date.'
                            })
                self.assertDictEqual(network.routes('salt', test='a'), ret)

                ret.update({'comment': 'Interface salt routes are'
                            ' set to be added.',
                            'result': None})
                self.assertDictEqual(network.routes('salt', test='a'), ret)

                ret.update({'comment': 'Interface salt routes are set to be'
                            ' updated:\n--- \n+++ \n@@ -1,4 +0,0 @@\n-T\n-r'
                            '\n-u\n-e'})
                self.assertDictEqual(network.routes('salt', test='a'), ret)

                mock = MagicMock(side_effect=[AttributeError, True])
                with patch.dict(network.__salt__,
                                {"ip.apply_network_settings": mock}):
                    ret.update({'changes': {'network_routes':
                                            'Added interface salt routes.'},
                                'comment': '',
                                'result': False})
                    self.assertDictEqual(network.routes('salt'), ret)

                    ret.update({'changes': {'network_routes':
                                            'Added interface salt routes.'},
                                'comment': 'Interface salt routes added.',
                                'result': True})
                    self.assertDictEqual(network.routes('salt'), ret)

    def test_system(self):
        '''
            Test to ensure that global network settings
            are configured properly
        '''
        ret = {'name': 'salt',
               'changes': {},
               'result': False,
               'comment': ''}

        with patch.dict(network.__opts__, {"test": True}):
            mock = MagicMock(side_effect=[AttributeError, False, False, 'As'])
            with patch.dict(network.__salt__,
                            {"ip.get_network_settings": mock}):
                self.assertDictEqual(network.system('salt'), ret)

                mock = MagicMock(side_effect=[False, True, ''])
                with patch.dict(network.__salt__,
                                {"ip.build_network_settings": mock}):
                    ret.update({'comment': 'Global network settings'
                                ' are up to date.',
                                'result': True})
                    self.assertDictEqual(network.system('salt'), ret)

                    ret.update({'comment': 'Global network settings are set to'
                                ' be added.',
                                'result': None})
                    self.assertDictEqual(network.system('salt'), ret)

                    ret.update({'comment': 'Global network settings are set to'
                                ' be updated:\n--- \n+++ \n@@ -1,2 +0,0'
                                ' @@\n-A\n-s'})
                    self.assertDictEqual(network.system('salt'), ret)

        with patch.dict(network.__opts__, {"test": False}):
            mock = MagicMock(side_effect=[False, False])
            with patch.dict(network.__salt__,
                            {"ip.get_network_settings": mock}):
                mock = MagicMock(side_effect=[True, True])
                with patch.dict(network.__salt__,
                                {"ip.build_network_settings": mock}):
                    mock = MagicMock(side_effect=[AttributeError, True])
                    with patch.dict(network.__salt__,
                                    {"ip.apply_network_settings": mock}):
                        ret.update({'changes': {'network_settings':
                                                'Added global network'
                                                ' settings.'},
                                    'comment': '',
                                    'result': False})
                        self.assertDictEqual(network.system('salt'), ret)

                        ret.update({'changes': {'network_settings':
                                                'Added global network'
                                                ' settings.'},
                                    'comment': 'Global network settings'
                                    ' are up to date.',
                                    'result': True})
                        self.assertDictEqual(network.system('salt'), ret)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(NetworkTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import twilio_notify


class MockTwilioRestException(Exception):
    '''
    Mock TwilioRestException class
    '''
    def __init__(self):
        self.code = 'error code'
        self.msg = 'Exception error'
        self.status = 'Not send'
        super(MockTwilioRestException, self).__init__(self.msg)


class MockMessages(object):
    '''
    Mock SMS class
    '''
    flag = None

    def __init__(self):
        self.sid = '011'
        self.price = '200'
        self.price_unit = '1'
        self.status = 'Sent'
        self.num_segments = '2'
        self.body = None
        self.date_sent = '01-01-2015'
        self.date_created = '01-01-2015'
        self.to = None
        self.from_ = None

    def create(self, body, to, from_):
        '''
        Mock create method
        '''
        msg = MockMessages()
        if self.flag == 1:
            raise MockTwilioRestException()
        msg.body = body
        msg.to = to
        msg.from_ = from_
        return msg


class MockSMS(object):
    '''
    Mock SMS class
    '''
    def __init__(self):
        self.messages = MockMessages()


class MockTwilioRestClient(object):
    '''
    Mock TwilioRestClient class
    '''
    def __init__(self):
        self.sms = MockSMS()

twilio_notify.TwilioRestClient = MockTwilioRestClient
twilio_notify.TwilioRestException = MockTwilioRestException


@skipIf(NO_MOCK, NO_MOCK_REASON)
class TwilioNotifyTestCase(TestCase):
    '''
    Test cases for salt.modules.twilio_notify
    '''
    # 'send_sms' function tests: 1

    def test_send_sms(self):
        '''
        Test if it send an sms.
        '''
        mock = MagicMock(return_value=MockTwilioRestClient())
        with patch.object(twilio_notify, '_get_twilio', mock):
            self.assertDictEqual(twilio_notify.send_sms('twilio-account',
                                                        'SALTSTACK',
                                                        '+18019999999',
                                                        '+18011111111'),
                                 {'message': {'status': 'Sent',
                                              'num_segments': '2',
                                              'price': '200',
                                              'body': 'SALTSTACK', 'sid': '011',
                                              'date_sent': '01-01-2015',
                                              'date_created': '01-01-2015',
                                              'price_unit': '1'}})

            MockMessages.flag = 1
            self.assertDictEqual(twilio_notify.send_sms('twilio-account',
                                                        'SALTSTACK',
                                                        '+18019999999',
                                                        '+18011111111'),
                                 {'message': {'sid': None}, '_error':
                                  {'msg': 'Exception error',
                                   'status': 'Not send', 'code': 'error code'}})


if __name__ == '__main__':
    from integration import run_tests
    run_tests(TwilioNotifyTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Module for managing timezone on POSIX-like systems.
'''
from __future__ import absolute_import

# Import python libs
import os
import errno
import logging
import re
import string

# Import salt libs
import salt.utils
from salt.exceptions import SaltInvocationError, CommandExecutionError

log = logging.getLogger(__name__)


def __virtual__():
    '''
    Only work on POSIX-like systems
    '''
    if salt.utils.is_windows():
        return False
    return True


def _timedatectl():
    '''
    get the output of timedatectl
    '''
    ret = __salt__['cmd.run_all'](['timedatectl'], python_shell=False)

    if ret['retcode'] != 0:
        msg = 'timedatectl failed: {0}'.format(ret['stderr'])
        raise CommandExecutionError(msg)

    return ret


def _get_zone_solaris():
    tzfile = '/etc/TIMEZONE'
    with salt.utils.fopen(tzfile, 'r') as fp_:
        for line in fp_:
            if 'TZ=' in line:
                zonepart = line.rstrip('\n').split('=')[-1]
                return zonepart.strip('\'"') or 'UTC'
    raise CommandExecutionError('Unable to get timezone from ' + tzfile)


def _get_zone_sysconfig():
    tzfile = '/etc/sysconfig/clock'
    with salt.utils.fopen(tzfile, 'r') as fp_:
        for line in fp_:
            if re.match(r'^\s*#', line):
                continue
            if 'ZONE' in line and '=' in line:
                zonepart = line.rstrip('\n').split('=')[-1]
                return zonepart.strip('\'"') or 'UTC'
    raise CommandExecutionError('Unable to get timezone from ' + tzfile)


def _get_zone_etc_localtime():
    tzfile = '/etc/localtime'
    tzdir = '/usr/share/zoneinfo/'
    tzdir_len = len(tzdir)
    try:
        olson_name = os.path.normpath(
            os.path.join('/etc', os.readlink(tzfile))
        )
        if olson_name.startswith(tzdir):
            return olson_name[tzdir_len:]
    except OSError as exc:
        if exc.errno == errno.ENOENT:
            raise CommandExecutionError(tzfile + ' does not exist')
        elif exc.errno == errno.EINVAL:
            log.warning(
                tzfile + ' is not a symbolic link, attempting to match ' +
                tzfile + ' to zoneinfo files'
            )
            # Regular file. Try to match the hash.
            hash_type = __opts__.get('hash_type', 'md5')
            tzfile_hash = salt.utils.get_hash(tzfile, hash_type)
            # Not a link, just a copy of the tzdata file
            for root, dirs, files in os.walk(tzdir):
                for filename in files:
                    full_path = os.path.join(root, filename)
                    olson_name = full_path[tzdir_len:]
                    if olson_name[0] in string.ascii_lowercase:
                        continue
                    if tzfile_hash == \
                            salt.utils.get_hash(full_path, hash_type):
                        return olson_name
    raise CommandExecutionError('Unable to determine timezone')


def _get_zone_etc_timezone():
    with salt.utils.fopen('/etc/timezone', 'r') as fp_:
        return fp_.read().strip()


def get_zone():
    '''
    Get current timezone (i.e. America/Denver)

    CLI Example:

    .. code-block:: bash

        salt '*' timezone.get_zone
    '''
    cmd = ''
    if salt.utils.which('timedatectl'):
        ret = _timedatectl()

        for line in (x.strip() for x in ret['stdout'].splitlines()):
            try:
                return re.match(r'Time ?zone:\s+(\S+)', line).group(1)
            except AttributeError:
                pass

        msg = ('Failed to parse timedatectl output: {0}\n'
               'Please file an issue with SaltStack').format(ret['stdout'])
        raise CommandExecutionError(msg)

    else:
        if __grains__['os'].lower() == 'centos':
            return _get_zone_etc_localtime()
        os_family = __grains__['os_family']
        for family in ('RedHat', 'Suse'):
            if family in os_family:
                return _get_zone_sysconfig()
        for family in ('Debian', 'Gentoo'):
            if family in os_family:
                return _get_zone_etc_timezone()
        if os_family in ('FreeBSD', 'OpenBSD', 'NetBSD'):
            return _get_zone_etc_localtime()
        elif 'Solaris' in os_family:
            return _get_zone_solaris()
    raise CommandExecutionError('Unable to get timezone')


def get_zonecode():
    '''
    Get current timezone (i.e. PST, MDT, etc)

    CLI Example:

    .. code-block:: bash

        salt '*' timezone.get_zonecode
    '''
    return __salt__['cmd.run'](['date', '+%Z'], python_shell=False)


def get_offset():
    '''
    Get current numeric timezone offset from UCT (i.e. -0700)

    CLI Example:

    .. code-block:: bash

        salt '*' timezone.get_offset
    '''
    return __salt__['cmd.run'](['date', '+%z'], python_shell=False)


def set_zone(timezone):
    '''
    Unlinks, then symlinks /etc/localtime to the set timezone.

    The timezone is crucial to several system processes, each of which SHOULD
    be restarted (for instance, whatever you system uses as its cron and
    syslog daemons). This will not be automagically done and must be done
    manually!

    CLI Example:

    .. code-block:: bash

        salt '*' timezone.set_zone 'America/Denver'
    '''
    if salt.utils.which('timedatectl'):
        try:
            __salt__['cmd.run']('timedatectl set-timezone {0}'.format(timezone))
        except CommandExecutionError:
            pass

    if 'Solaris' in __grains__['os_family']:
        zonepath = '/usr/share/lib/zoneinfo/{0}'.format(timezone)
    else:
        zonepath = '/usr/share/zoneinfo/{0}'.format(timezone)
    if not os.path.exists(zonepath):
        return 'Zone does not exist: {0}'.format(zonepath)

    if os.path.exists('/etc/localtime'):
        os.unlink('/etc/localtime')

    if 'Solaris' in __grains__['os_family']:
        __salt__['file.sed'](
            '/etc/default/init', '^TZ=.*', 'TZ={0}'.format(timezone))
    else:
        os.symlink(zonepath, '/etc/localtime')

    if 'RedHat' in __grains__['os_family']:
        __salt__['file.sed'](
            '/etc/sysconfig/clock', '^ZONE=.*', 'ZONE="{0}"'.format(timezone))
    elif 'Suse' in __grains__['os_family']:
        __salt__['file.sed'](
            '/etc/sysconfig/clock', '^ZONE=.*', 'ZONE="{0}"'.format(timezone))
    elif 'Debian' in __grains__['os_family']:
        with salt.utils.fopen('/etc/timezone', 'w') as ofh:
            ofh.write(timezone.strip())
            ofh.write('\n')
    elif 'Gentoo' in __grains__['os_family']:
        with salt.utils.fopen('/etc/timezone', 'w') as ofh:
            ofh.write(timezone)

    return True


def zone_compare(timezone):
    '''
    Checks the hash sum between the given timezone, and the one set in
    /etc/localtime. Returns True if they match, and False if not. Mostly useful
    for running state checks.

    CLI Example:

    .. code-block:: bash

        salt '*' timezone.zone_compare 'America/Denver'
    '''
    if 'Solaris' in __grains__['os_family']:
        return 'Not implemented for Solaris family'

    tzfile = '/etc/localtime'
    zonepath = '/usr/share/zoneinfo/{0}'.format(timezone)

    if not os.path.exists(tzfile):
        return 'Error: {0} does not exist.'.format(tzfile)

    hash_type = __opts__.get('hash_type', 'md5')

    try:
        usrzone = salt.utils.get_hash(zonepath, hash_type)
    except IOError as exc:
        raise SaltInvocationError('Invalid timezone {0!r}'.format(timezone))

    try:
        etczone = salt.utils.get_hash(tzfile, hash_type)
    except IOError as exc:
        raise CommandExecutionError(
            'Problem reading timezone file {0}: {1}'
            .format(tzfile, exc.strerror)
        )

    if usrzone == etczone:
        return True
    return False


def get_hwclock():
    '''
    Get current hardware clock setting (UTC or localtime)

    CLI Example:

    .. code-block:: bash

        salt '*' timezone.get_hwclock
    '''
    cmd = ''
    if salt.utils.which('timedatectl'):
        ret = _timedatectl()
        for line in (x.strip() for x in ret['stdout'].splitlines()):
            if 'rtc in local tz' in line.lower():
                try:
                    if line.split(':')[-1].strip().lower() == 'yes':
                        return 'localtime'
                    else:
                        return 'UTC'
                except IndexError:
                    pass

        msg = ('Failed to parse timedatectl output: {0}\n'
               'Please file an issue with SaltStack').format(ret['stdout'])
        raise CommandExecutionError(msg)

    else:
        os_family = __grains__['os_family']
        for family in ('RedHat', 'Suse'):
            if family in os_family:
                cmd = ['tail', '-n', '1', '/etc/adjtime']
                return __salt__['cmd.run'](cmd, python_shell=False)
        if 'Debian' in __grains__['os_family']:
            # Original way to look up hwclock on Debian-based systems
            try:
                with salt.utils.fopen('/etc/default/rcS', 'r') as fp_:
                    for line in fp_:
                        if re.match(r'^\s*#', line):
                            continue
                        if 'UTC=' in line:
                            is_utc = line.rstrip('\n').split('=')[-1].lower()
                            if is_utc == 'yes':
                                return 'UTC'
                            else:
                                return 'localtime'
            except IOError as exc:
                pass
            # Since Wheezy
            cmd = ['tail', '-n', '1', '/etc/adjtime']
            return __salt__['cmd.run'](cmd, python_shell=False)
        elif 'Gentoo' in __grains__['os_family']:
            offset_file = '/etc/conf.d/hwclock'
            try:
                with salt.utils.fopen(offset_file, 'r') as fp_:
                    for line in fp_:
                        if line.startswith('clock='):
                            line = line.rstrip('\n')
                            return line.split('=')[-1].strip('\'"')
                    raise CommandExecutionError(
                        'Offset information not found in {0}'.format(
                            offset_file
                        )
                    )
            except IOError as exc:
                raise CommandExecutionError(
                    'Problem reading offset file {0}: {1}'
                    .format(offset_file, exc.strerror)
                )
        elif 'Solaris' in __grains__['os_family']:
            offset_file = '/etc/rtc_config'
            try:
                with salt.utils.fopen(offset_file, 'r') as fp_:
                    for line in fp_:
                        if line.startswith('zone_info=GMT'):
                            return 'UTC'
                    return 'localtime'
            except IOError as exc:
                if exc.errno == errno.ENOENT:
                    # offset file does not exist
                    return 'UTC'
                raise CommandExecutionError(
                    'Problem reading offset file {0}: {1}'
                    .format(offset_file, exc.strerror)
                )


def set_hwclock(clock):
    '''
    Sets the hardware clock to be either UTC or localtime

    CLI Example:

    .. code-block:: bash

        salt '*' timezone.set_hwclock UTC
    '''
    timezone = get_zone()

    if 'Solaris' in __grains__['os_family']:
        if clock.lower() not in ('localtime', 'utc'):
            raise SaltInvocationError(
                'localtime and UTC are the only permitted values'
            )
        if 'sparc' in __grains__['cpuarch']:
            raise SaltInvocationError(
                'UTC is the only choice for SPARC architecture'
            )
        cmd = ['rtc', '-z', 'GMT' if clock.lower() == 'utc' else timezone]
        return __salt__['cmd.retcode'](cmd, python_shell=False) == 0

    zonepath = '/usr/share/zoneinfo/{0}'.format(timezone)

    if not os.path.exists(zonepath):
        raise CommandExecutionError(
            'Zone \'{0}\' does not exist'.format(zonepath)
        )

    os.unlink('/etc/localtime')
    os.symlink(zonepath, '/etc/localtime')

    if 'Arch' in __grains__['os_family']:
        cmd = ['timezonectl', 'set-local-rtc',
               'true' if clock == 'localtime' else 'false']
        return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
    elif 'RedHat' in __grains__['os_family']:
        __salt__['file.sed'](
            '/etc/sysconfig/clock', '^ZONE=.*', 'ZONE="{0}"'.format(timezone))
    elif 'Suse' in __grains__['os_family']:
        __salt__['file.sed'](
            '/etc/sysconfig/clock', '^ZONE=.*', 'ZONE="{0}"'.format(timezone))
    elif 'Debian' in __grains__['os_family']:
        if clock == 'UTC':
            __salt__['file.sed']('/etc/default/rcS', '^UTC=.*', 'UTC=yes')
        elif clock == 'localtime':
            __salt__['file.sed']('/etc/default/rcS', '^UTC=.*', 'UTC=no')
    elif 'Gentoo' in __grains__['os_family']:
        if clock not in ('UTC', 'localtime'):
            raise SaltInvocationError(
                'Only \'UTC\' and \'localtime\' are allowed'
            )
        if clock == 'localtime':
            clock = 'local'
        __salt__['file.sed'](
            '/etc/conf.d/hwclock', '^clock=.*', 'clock="{0}"'.format(clock))

    return True



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import
import uuid

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)
from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import iptables

# Globals
iptables.__grains__ = {}
iptables.__salt__ = {}
iptables.__context__ = {}
iptables.__opts__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class IptablesTestCase(TestCase):
    '''
    Test cases for salt.modules.iptables
    '''
    # 'version' function tests: 1

    def test_version(self):
        '''
        Test if it return version from iptables --version
        '''
        mock = MagicMock(return_value='iptables v1.4.21')
        with patch.dict(iptables.__salt__, {'cmd.run': mock}):
            self.assertEqual(iptables.version(), 'v1.4.21')

    # 'build_rule' function tests: 1

    @patch.object(iptables, '_has_option', MagicMock(return_value=True))
    def test_build_rule(self):
        '''
        Test if it build a well-formatted iptables rule based on kwargs.
        '''
        self.assertEqual(iptables.build_rule(), '')

        self.assertEqual(iptables.build_rule(name='ignored', state='ignored'),
                         '',
                         'build_rule should ignore name and state')

        # Should properly negate bang-prefixed values
        self.assertEqual(iptables.build_rule(**{'if': '!eth0'}),
                         '! -i eth0')

        # Should properly negate "not"-prefixed values
        self.assertEqual(iptables.build_rule(**{'if': 'not eth0'}),
                         '! -i eth0')

        self.assertEqual(iptables.build_rule(dports=[80, 443], proto='tcp'),
                         '-p tcp -m multiport --dports 80,443')

        self.assertEqual(iptables.build_rule(dports='80,443', proto='tcp'),
                         '-p tcp -m multiport --dports 80,443')

        # Should it really behave this way?
        self.assertEqual(iptables.build_rule(dports=['!80', 443],
                                             proto='tcp'),
                         '-p tcp -m multiport ! --dports 80,443')

        self.assertEqual(iptables.build_rule(dports='!80,443', proto='tcp'),
                         '-p tcp -m multiport ! --dports 80,443')

        self.assertEqual(iptables.build_rule(sports=[80, 443], proto='tcp'),
                         '-p tcp -m multiport --sports 80,443')

        self.assertEqual(iptables.build_rule(sports='80,443', proto='tcp'),
                         '-p tcp -m multiport --sports 80,443')

        self.assertEqual(iptables.build_rule('filter', 'INPUT', command='I',
                                             position='3', full=True,
                                             dports='proto', jump='ACCEPT'),
                         'Error: proto must be specified')

        self.assertEqual(iptables.build_rule('filter', 'INPUT', command='I',
                                             position='3', full=True,
                                             sports='proto', jump='ACCEPT'),
                         'Error: proto must be specified')

        self.assertEqual(iptables.build_rule('', 'INPUT', command='I',
                                             position='3', full='True',
                                             match='state', jump='ACCEPT'),
                         'Error: Table needs to be specified')

        self.assertEqual(iptables.build_rule('filter', '', command='I',
                                             position='3', full='True',
                                             match='state', jump='ACCEPT'),
                         'Error: Chain needs to be specified')

        self.assertEqual(iptables.build_rule('filter', 'INPUT', command='',
                                             position='3', full='True',
                                             match='state', jump='ACCEPT'),
                         'Error: Command needs to be specified')

        # Test arguments that should appear after the --jump
        self.assertEqual(iptables.build_rule(jump='REDIRECT',
                                             **{'to-port': 8080}),
                         '--jump REDIRECT --to-port 8080')

        # Should quote arguments with spaces, like log-prefix often has
        self.assertEqual(iptables.build_rule(jump='LOG',
                                             **{'log-prefix': 'long prefix'}),
                         '--jump LOG --log-prefix "long prefix"')

        # Should quote arguments with leading or trailing spaces
        self.assertEqual(iptables.build_rule(jump='LOG',
                                             **{'log-prefix': 'spam: '}),
                         '--jump LOG --log-prefix "spam: "')

        # Should allow no-arg jump options
        self.assertEqual(iptables.build_rule(jump='CLUSTERIP',
                                             **{'new': ''}),
                         '--jump CLUSTERIP --new ')

        # should build match-sets with single string
        self.assertEqual(iptables.build_rule(**{'match-set': 'src flag1,flag2'}),
                         '-m set --match-set src flag1,flag2')

        # should build match-sets as list
        match_sets = ['src1 flag1',
                      'src2 flag2,flag3',
                     ]
        self.assertEqual(iptables.build_rule(**{'match-set': match_sets}),
                         '-m set --match-set src1 flag1 -m set --match-set src2 flag2,flag3')

        # should handle negations for string match-sets
        self.assertEqual(iptables.build_rule(**{'match-set': '!src flag'}),
                         '-m set ! --match-set src flag')

        # should handle negations for list match-sets
        match_sets = ['src1 flag',
                      'not src2 flag2']
        self.assertEqual(iptables.build_rule(**{'match-set': match_sets}),
                         '-m set --match-set src1 flag -m set ! --match-set src2 flag2')

        # Should allow the --save jump option to CONNSECMARK
        #self.assertEqual(iptables.build_rule(jump='CONNSECMARK',
        #                                     **{'save': ''}),
        #                 '--jump CONNSECMARK --save ')

        ret = '/sbin/iptables --wait -t salt -I INPUT 3 -m state --jump ACCEPT'
        with patch.object(iptables, '_iptables_cmd',
                          MagicMock(return_value='/sbin/iptables')):
            self.assertEqual(iptables.build_rule('salt', 'INPUT', command='I',
                                                 position='3', full='True',
                                                 match='state', jump='ACCEPT'),
                             ret)

    # 'get_saved_rules' function tests: 1

    def test_get_saved_rules(self):
        '''
        Test if it return a data structure of the rules in the conf file
        '''
        mock = MagicMock(return_value=False)
        with patch.object(iptables, '_parse_conf', mock):
            self.assertFalse(iptables.get_saved_rules())
            mock.assert_called_with(conf_file=None, family='ipv4')

    # 'get_rules' function tests: 1

    def test_get_rules(self):
        '''
        Test if it return a data structure of the current, in-memory rules
        '''
        mock = MagicMock(return_value=False)
        with patch.object(iptables, '_parse_conf', mock):
            self.assertFalse(iptables.get_rules())
            mock.assert_called_with(in_mem=True, family='ipv4')

    # 'get_saved_policy' function tests: 1

    def test_get_saved_policy(self):
        '''
        Test if it return the current policy for the specified table/chain
        '''
        self.assertEqual(iptables.get_saved_policy(table='filter', chain=None,
                                                   conf_file=None,
                                                   family='ipv4'),
                         'Error: Chain needs to be specified')

        with patch.object(iptables, '_parse_conf',
                          MagicMock(return_value={'filter':
                                                  {'INPUT':
                                                   {'policy': True}}})):
            self.assertTrue(iptables.get_saved_policy(table='filter',
                                                       chain='INPUT',
                                                       conf_file=None,
                                                       family='ipv4'))

        with patch.object(iptables, '_parse_conf',
                          MagicMock(return_value={'filter':
                                                  {'INPUT':
                                                   {'policy1': True}}})):
            self.assertIsNone(iptables.get_saved_policy(table='filter',
                                                       chain='INPUT',
                                                       conf_file=None,
                                                       family='ipv4'))

    # 'get_policy' function tests: 1

    def test_get_policy(self):
        '''
        Test if it return the current policy for the specified table/chain
        '''
        self.assertEqual(iptables.get_policy(table='filter', chain=None,
                                                   family='ipv4'),
                         'Error: Chain needs to be specified')

        with patch.object(iptables, '_parse_conf',
                          MagicMock(return_value={'filter':
                                                  {'INPUT':
                                                   {'policy': True}}})):
            self.assertTrue(iptables.get_policy(table='filter',
                                                       chain='INPUT',
                                                       family='ipv4'))

        with patch.object(iptables, '_parse_conf',
                          MagicMock(return_value={'filter':
                                                  {'INPUT':
                                                   {'policy1': True}}})):
            self.assertIsNone(iptables.get_policy(table='filter',
                                                       chain='INPUT',
                                                       family='ipv4'))

    # 'set_policy' function tests: 1

    @patch.object(iptables, '_has_option', MagicMock(return_value=True))
    def test_set_policy(self):
        '''
        Test if it set the current policy for the specified table/chain
        '''
        self.assertEqual(iptables.set_policy(table='filter', chain=None,
                                                   policy=None,
                                                   family='ipv4'),
                         'Error: Chain needs to be specified')

        self.assertEqual(iptables.set_policy(table='filter', chain='INPUT',
                                                   policy=None,
                                                   family='ipv4'),
                         'Error: Policy needs to be specified')

        mock = MagicMock(return_value=True)
        with patch.dict(iptables.__salt__, {'cmd.run': mock}):
            self.assertTrue(iptables.set_policy(table='filter',
                                                       chain='INPUT',
                                                       policy='ACCEPT',
                                                       family='ipv4'))

    # 'save' function tests: 1

    @patch('salt.modules.iptables._conf', MagicMock(return_value=False))
    @patch('os.path.isdir', MagicMock(return_value=True))
    def test_save(self):
        '''
        Test if it save the current in-memory rules to disk
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(iptables.__salt__, {'cmd.run': mock,
                                            'file.write': mock}):
            self.assertTrue(iptables.save(filename='/xyz', family='ipv4'))

    # 'check' function tests: 1

    def test_check(self):
        '''
        Test if it check for the existence of a rule in the table and chain
        '''
        self.assertEqual(iptables.check(table='filter', chain=None,
                                                   rule=None,
                                                   family='ipv4'),
                         'Error: Chain needs to be specified')

        self.assertEqual(iptables.check(table='filter', chain='INPUT',
                                                   rule=None,
                                                   family='ipv4'),
                         'Error: Rule needs to be specified')

        mock_rule = 'm state --state RELATED,ESTABLISHED -j ACCEPT'
        mock_chain = 'INPUT'
        mock_uuid = 31337
        mock_cmd = MagicMock(return_value='-A {0}\n-A {1}'.format(mock_chain,
                                                                 hex(mock_uuid)))
        mock_has = MagicMock(return_value=True)
        mock_not = MagicMock(return_value=False)

        with patch.object(iptables, '_has_option', mock_not):
            with patch.object(uuid, 'getnode', MagicMock(return_value=mock_uuid)):
                with patch.dict(iptables.__salt__, {'cmd.run': mock_cmd}):
                    self.assertTrue(iptables.check(table='filter', chain=mock_chain,
                                                   rule=mock_rule, family='ipv4'))

        mock_cmd = MagicMock(return_value='')

        with patch.object(iptables, '_has_option', mock_not):
            with patch.object(uuid, 'getnode', MagicMock(return_value=mock_uuid)):
                with patch.dict(iptables.__salt__, {'cmd.run': MagicMock(return_value='')}):
                    self.assertFalse(iptables.check(table='filter', chain=mock_chain,
                                                    rule=mock_rule, family='ipv4'))

        with patch.object(iptables, '_has_option', mock_has):
            with patch.dict(iptables.__salt__, {'cmd.run': mock_cmd}):
                self.assertTrue(iptables.check(table='filter', chain='INPUT',
                                               rule=mock_rule, family='ipv4'))

        mock_cmd = MagicMock(return_value='-A 0x4d2')
        mock_uuid = MagicMock(return_value=1234)

        with patch.object(iptables, '_has_option', mock_has):
            with patch.object(uuid, 'getnode', mock_uuid):
                with patch.dict(iptables.__salt__, {'cmd.run': mock_cmd}):
                    self.assertTrue(iptables.check(table='filter',
                                                   chain='0x4d2',
                                                   rule=mock_rule, family='ipv4'))

    # 'check_chain' function tests: 1

    def test_check_chain(self):
        '''
        Test if it check for the existence of a chain in the table
        '''
        self.assertEqual(iptables.check_chain(table='filter', chain=None,
                                                   family='ipv4'),
                         'Error: Chain needs to be specified')

        mock_cmd = MagicMock(return_value='')
        with patch.dict(iptables.__salt__, {'cmd.run': mock_cmd}):
            self.assertFalse(iptables.check_chain(table='filter',
                                                       chain='INPUT',
                                                       family='ipv4'))

    # 'new_chain' function tests: 1

    def test_new_chain(self):
        '''
        Test if it create new custom chain to the specified table.
        '''
        self.assertEqual(iptables.new_chain(table='filter', chain=None,
                                                   family='ipv4'),
                         'Error: Chain needs to be specified')

        mock_cmd = MagicMock(return_value='')
        with patch.dict(iptables.__salt__, {'cmd.run': mock_cmd}):
            self.assertTrue(iptables.new_chain(table='filter',
                                                       chain='INPUT',
                                                       family='ipv4'))

    # 'delete_chain' function tests: 1

    def test_delete_chain(self):
        '''
        Test if it delete custom chain to the specified table.
        '''
        self.assertEqual(iptables.delete_chain(table='filter', chain=None,
                                                   family='ipv4'),
                         'Error: Chain needs to be specified')

        mock_cmd = MagicMock(return_value='')
        with patch.dict(iptables.__salt__, {'cmd.run': mock_cmd}):
            self.assertTrue(iptables.delete_chain(table='filter',
                                                       chain='INPUT',
                                                       family='ipv4'))

    # 'append' function tests: 1

    @patch.object(iptables, '_has_option', MagicMock(return_value=True))
    def test_append(self):
        '''
        Test if it append a rule to the specified table/chain.
        '''
        self.assertEqual(iptables.append(table='filter', chain=None,
                                                   rule=None,
                                                   family='ipv4'),
                         'Error: Chain needs to be specified')

        self.assertEqual(iptables.append(table='filter', chain='INPUT',
                                                   rule=None,
                                                   family='ipv4'),
                         'Error: Rule needs to be specified')

        _rule = 'm state --state RELATED,ESTABLISHED -j ACCEPT'
        mock = MagicMock(side_effect=['', 'SALT'])
        with patch.dict(iptables.__salt__, {'cmd.run': mock}):
            self.assertTrue(iptables.append(table='filter', chain='INPUT',
                                            rule=_rule, family='ipv4'))

            self.assertFalse(iptables.append(table='filter', chain='INPUT',
                                            rule=_rule, family='ipv4'))

    # 'insert' function tests: 1

    @patch.object(iptables, '_has_option', MagicMock(return_value=True))
    def test_insert(self):
        '''
        Test if it insert a rule into the specified table/chain,
        at the specified position.
        '''
        self.assertEqual(iptables.insert(table='filter', chain=None,
                                         position=None, rule=None,
                                                   family='ipv4'),
                         'Error: Chain needs to be specified')

        pos_err = 'Error: Position needs to be specified or use append (-A)'
        self.assertEqual(iptables.insert(table='filter', chain='INPUT',
                                         position=None, rule=None,
                                                   family='ipv4'), pos_err)

        self.assertEqual(iptables.insert(table='filter', chain='INPUT',
                                         position=3, rule=None,
                                                   family='ipv4'),
                         'Error: Rule needs to be specified')

        _rule = 'm state --state RELATED,ESTABLISHED -j ACCEPT'
        mock = MagicMock(return_value=True)
        with patch.dict(iptables.__salt__, {'cmd.run': mock}):
            self.assertTrue(iptables.insert(table='filter', chain='INPUT',
                                            position=3, rule=_rule,
                                            family='ipv4'))

    # 'delete' function tests: 1

    @patch.object(iptables, '_has_option', MagicMock(return_value=True))
    def test_delete(self):
        '''
        Test if it delete a rule from the specified table/chain
        '''
        _rule = 'm state --state RELATED,ESTABLISHED -j ACCEPT'
        self.assertEqual(iptables.delete(table='filter', chain=None,
                                         position=3, rule=_rule,
                                                   family='ipv4'),
                         'Error: Only specify a position or a rule, not both')

        mock = MagicMock(return_value=True)
        with patch.dict(iptables.__salt__, {'cmd.run': mock}):
            self.assertTrue(iptables.delete(table='filter', chain='INPUT',
                                            position=3, rule='',
                                            family='ipv4'))

    # 'flush' function tests: 1

    @patch.object(iptables, '_has_option', MagicMock(return_value=True))
    def test_flush(self):
        '''
        Test if it flush the chain in the specified table,
        flush all chains in the specified table if not specified chain.
        '''
        mock_cmd = MagicMock(return_value=True)
        with patch.dict(iptables.__salt__, {'cmd.run': mock_cmd}):
            self.assertTrue(iptables.flush(table='filter',
                                                       chain='INPUT',
                                                       family='ipv4'))


if __name__ == '__main__':
    from integration import run_tests
    run_tests(IptablesTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import absolute_import
import os

from salt.utils import parsers
from salt.utils.verify import verify_log
from salt.config import _expand_glob_path
import salt.cli.caller
import salt.defaults.exitcodes


class SaltCall(parsers.SaltCallOptionParser):
    '''
    Used to locally execute a salt command
    '''

    def run(self):
        '''
        Execute the salt call!
        '''
        self.parse_args()

        if self.options.file_root:
            # check if the argument is pointing to a file on disk
            file_root = os.path.abspath(self.options.file_root)
            self.config['file_roots'] = {'base': _expand_glob_path([file_root])}

        if self.options.pillar_root:
            # check if the argument is pointing to a file on disk
            pillar_root = os.path.abspath(self.options.pillar_root)
            self.config['pillar_roots'] = {'base': _expand_glob_path([pillar_root])}

        if self.options.local:
            self.config['file_client'] = 'local'
        if self.options.master:
            self.config['master'] = self.options.master

        # Setup file logging!
        self.setup_logfile_logger()
        verify_log(self.config)

        caller = salt.cli.caller.Caller.factory(self.config)

        if self.options.doc:
            caller.print_docs()
            self.exit(salt.defaults.exitcodes.EX_OK)

        if self.options.grains_run:
            caller.print_grains()
            self.exit(salt.defaults.exitcodes.EX_OK)

        caller.run()



# -*- coding: utf-8 -*-
'''
Return salt data via slack

.. versionadded:: 2015.5.0

The following fields can be set in the minion conf file::

    slack.channel (required)
    slack.api_key (required)
    slack.username (required)
    slack.as_user (required to see the profile picture of your bot)
    slack.profile (optional)

Alternative configuration values can be used by prefacing the configuration.
Any values not found in the alternative configuration will be pulled from
the default location::

    slack.channel
    slack.api_key
    slack.username
    slack.as_user

Slack settings may also be configured as::

    slack:
        channel: RoomName
        api_key: xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
        username: user
        as_user: true

    alternative.slack:
        room_id: RoomName
        api_key: xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
        from_name: user@email.com

    slack_profile:
        api_key: xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
        from_name: user@email.com

    slack:
        profile: slack_profile
        channel: RoomName

    alternative.slack:
        profile: slack_profile
        channel: RoomName

To use the Slack returner, append '--return slack' to the salt command.

.. code-block:: bash

    salt '*' test.ping --return slack

To use the alternative configuration, append '--return_config alternative' to the salt command.

.. code-block:: bash

    salt '*' test.ping --return slack --return_config alternative
'''
from __future__ import absolute_import

# Import Python libs
import pprint
import logging
import urllib

# pylint: disable=import-error,no-name-in-module,redefined-builtin
from salt.ext.six.moves.urllib.parse import urljoin as _urljoin  # pylint: disable=import-error,no-name-in-module
import salt.ext.six.moves.http_client
# pylint: enable=import-error,no-name-in-module,redefined-builtin

# Import Salt Libs
import salt.returners

log = logging.getLogger(__name__)

__virtualname__ = 'slack'


def _get_options(ret=None):
    '''
    Get the slack options from salt.
    '''

    defaults = {'channel': '#general'}

    attrs = {'slack_profile': 'profile',
             'channel': 'channel',
             'username': 'username',
             'as_user': 'as_user',
             'api_key': 'api_key',
             }

    profile_attr = 'slack_profile'

    profile_attrs = {'from_jid': 'from_jid',
                     'api_key': 'api_key',
                     'api_version': 'api_key'
                     }

    _options = salt.returners.get_returner_options(__virtualname__,
                                                   ret,
                                                   attrs,
                                                   profile_attr=profile_attr,
                                                   profile_attrs=profile_attrs,
                                                   __salt__=__salt__,
                                                   __opts__=__opts__,
                                                   defaults=defaults)
    return _options


def __virtual__():
    '''
    Return virtual name of the module.

    :return: The virtual name of the module.
    '''
    return __virtualname__


def _query(function,
           api_key=None,
           args=None,
           method='GET',
           header_dict=None,
           data=None):
    '''
    Slack object method function to construct and execute on the API URL.

    :param api_key:     The Slack api key.
    :param function:    The Slack api function to perform.
    :param method:      The HTTP method, e.g. GET or POST.
    :param data:        The data to be sent for POST method.
    :return:            The json response from the API call or False.
    '''
    query_params = {}

    ret = {'message': '',
           'res': True}

    slack_functions = {
        'rooms': {
            'request': 'channels.list',
            'response': 'channels',
        },
        'users': {
            'request': 'users.list',
            'response': 'members',
        },
        'message': {
            'request': 'chat.postMessage',
            'response': 'channel',
        },
    }

    if not api_key:
        try:
            options = __salt__['config.option']('slack')
            if not api_key:
                api_key = options.get('api_key')
        except (NameError, KeyError, AttributeError):
            log.error('No Slack api key found.')
            ret['message'] = 'No Slack api key found.'
            ret['res'] = False
            return ret

    api_url = 'https://slack.com'
    base_url = _urljoin(api_url, '/api/')
    path = slack_functions.get(function).get('request')
    url = _urljoin(base_url, path, False)

    if not isinstance(args, dict):
        query_params = {}
    query_params['token'] = api_key

    if header_dict is None:
        header_dict = {}

    if method != 'POST':
        header_dict['Accept'] = 'application/json'

    result = salt.utils.http.query(
        url,
        method,
        params=query_params,
        data=data,
        decode=True,
        status=True,
        header_dict=header_dict,
        opts=__opts__,
    )

    if result.get('status', None) == salt.ext.six.moves.http_client.OK:
        _result = result['dict']
        response = slack_functions.get(function).get('response')
        if 'error' in _result:
            ret['message'] = _result['error']
            ret['res'] = False
            return ret
        ret['message'] = _result.get(response)
        return ret
    elif result.get('status', None) == salt.ext.six.moves.http_client.NO_CONTENT:
        return True
    else:
        log.debug(url)
        log.debug(query_params)
        log.debug(data)
        log.debug(result)
        _result = result['dict']
        if 'error' in _result:
            ret['message'] = _result['error']
            ret['res'] = False
            return ret
        ret['message'] = _result.get(response)
        return ret


def _post_message(channel,
                  message,
                  username,
                  as_user,
                  api_key=None):
    '''
    Send a message to a Slack room.
    :param channel:     The room name.
    :param message:     The message to send to the Slack room.
    :param username:    Specify who the message is from.
    :param as_user:     Sets the profile picture which have been added through Slack itself.
    :param api_key:     The Slack api key, if not specified in the configuration.
    :param api_version: The Slack api version, if not specified in the configuration.
    :return:            Boolean if message was sent successfully.
    '''

    parameters = dict()
    parameters['channel'] = channel
    parameters['username'] = username
    parameters['as_user'] = as_user
    parameters['text'] = '```' + message + '```'  # pre-formatted, fixed-width text

    # Slack wants the body on POST to be urlencoded.
    result = _query(function='message',
                    api_key=api_key,
                    method='POST',
                    header_dict={'Content-Type': 'application/x-www-form-urlencoded'},
                    data=urllib.urlencode(parameters))

    log.debug('result {0}'.format(result))
    if result:
        return True
    else:
        return False


def returner(ret):
    '''
    Send an slack message with the data
    '''

    _options = _get_options(ret)

    channel = _options.get('channel')
    username = _options.get('username')
    as_user = _options.get('as_user')
    api_key = _options.get('api_key')

    if not channel:
        log.error('slack.channel not defined in salt config')
        return

    if not username:
        log.error('slack.username not defined in salt config')
        return

    if not as_user:
        log.error('slack.as_user not defined in salt config')
        return

    if not api_key:
        log.error('slack.api_key not defined in salt config')
        return

    message = ('id: {0}\r\n'
               'function: {1}\r\n'
               'function args: {2}\r\n'
               'jid: {3}\r\n'
               'return: {4}\r\n').format(
                    ret.get('id'),
                    ret.get('fun'),
                    ret.get('fun_args'),
                    ret.get('jid'),
                    pprint.pformat(ret.get('return')))

    slack = _post_message(channel,
                          message,
                          username,
                          as_user,
                          api_key)
    return slack



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import quota

quota.__opts__ = {}
quota.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class QuotaTestCase(TestCase):
    '''
    Test cases for salt.states.quota
    '''
    # 'mode' function tests: 1

    def test_mode(self):
        '''
        Test to set the quota for the system.
        '''
        name = '/'
        mode = True
        quotatype = 'user'

        ret = {'name': name,
               'changes': {},
               'result': False,
               'comment': ''}

        mock_bool = MagicMock(side_effect=[True, False])
        mock = MagicMock(return_value={name: {quotatype: 'on'}})
        with patch.dict(quota.__salt__, {'quota.get_mode': mock}):
            comt = ('Quota for / already set to on')
            ret.update({'comment': comt, 'result': True})
            self.assertDictEqual(quota.mode(name, mode, quotatype), ret)

        mock = MagicMock(return_value={name: {quotatype: 'off'}})
        with patch.dict(quota.__salt__, {'quota.get_mode': mock,
                                         'quota.on': mock_bool}):
            with patch.dict(quota.__opts__, {'test': True}):
                comt = ('Quota for / needs to be set to on')
                ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(quota.mode(name, mode, quotatype), ret)

            with patch.dict(quota.__opts__, {'test': False}):
                comt = ('Set quota for / to on')
                ret.update({'comment': comt, 'result': True,
                            'changes': {'quota': name}})
                self.assertDictEqual(quota.mode(name, mode, quotatype), ret)

                comt = ('Failed to set quota for / to on')
                ret.update({'comment': comt, 'result': False, 'changes': {}})
                self.assertDictEqual(quota.mode(name, mode, quotatype), ret)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(QuotaTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
sysrc module for FreeBSD
'''

# Import Python libs
from __future__ import absolute_import

# Import Salt libs
import salt.utils
from salt.exceptions import CommandExecutionError


__virtualname__ = 'sysrc'

__func_alias__ = {
    'set_': 'set'
}


def __virtual__():
    '''
    Only runs if sysrc exists
    '''
    if salt.utils.which('sysrc') is not None:
        return True
    return False


def get(**kwargs):
    '''
    Return system rc configuration variables

    CLI Example:

     .. code-block:: bash

         salt '*' sysrc.get includeDefaults=True
    '''

    cmd = 'sysrc -v'

    if 'file' in kwargs:
        cmd += ' -f '+kwargs['file']

    if 'jail' in kwargs:
        cmd += ' -j '+kwargs['jail']

    if 'name' in kwargs:
        cmd += ' '+kwargs['name']
    elif kwargs.get('includeDefaults', False):
        cmd += ' -A'
    else:
        cmd += ' -a'

    sysrcs = __salt__['cmd.run'](cmd)
    if "sysrc: unknown variable" in sysrcs:
        # raise CommandExecutionError(sysrcs)
        return None

    ret = {}
    for sysrc in sysrcs.split("\n"):
        line_components = sysrc.split(': ')
        rcfile = line_components[0]
        if len(line_components) > 2:
            var = line_components[1]
            val = line_components[2]
        else:
            var = line_components[1].rstrip(':')
            val = ''
        if rcfile not in ret:
            ret[rcfile] = {}
        ret[rcfile][var] = val
    return ret


def set_(name, value, **kwargs):
    '''
    Set system rc configuration variables

    CLI Example:

     .. code-block:: bash

         salt '*' sysrc.set name=sshd_flags value="-p 2222"
    '''

    cmd = 'sysrc -v'

    if 'file' in kwargs:
        cmd += ' -f '+kwargs['file']

    if 'jail' in kwargs:
        cmd += ' -j '+kwargs['jail']

    cmd += ' '+name+"=\""+value+"\""

    sysrcs = __salt__['cmd.run'](cmd)

    ret = {}
    for sysrc in sysrcs.split("\n"):
        rcfile = sysrc.split(': ')[0]
        var = sysrc.split(': ')[1]
        oldval = sysrc.split(': ')[2].split(" -> ")[0]
        newval = sysrc.split(': ')[2].split(" -> ")[1]
        if rcfile not in ret:
            ret[rcfile] = {}
        #ret[rcfile][var] = {}
        #ret[rcfile][var]['old'] = oldval
        #ret[rcfile][var]['new'] = newval
        ret[rcfile][var] = newval
    return ret


def remove(name, **kwargs):
    '''
    Remove system rc configuration variables

    CLI Example:

     .. code-block:: bash

         salt '*' sysrc.remove name=sshd_enable
    '''

    cmd = 'sysrc -v'

    if 'file' in kwargs:
        cmd += ' -f '+kwargs['file']

    if 'jail' in kwargs:
        cmd += ' -j '+kwargs['jail']

    cmd += ' -x '+name

    sysrcs = __salt__['cmd.run'](cmd)
    if "sysrc: unknown variable" in sysrcs:
        raise CommandExecutionError(sysrcs)
    else:
        return name+" removed"



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Erik Johnson <erik@saltstack.com>`
'''

# Import Python libs
from __future__ import absolute_import
import copy
import logging
import os
import subprocess
from distutils.version import LooseVersion

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
from salt.modules import git as git_mod  # Don't potentially shadow GitPython

# Globals
git_mod.__salt__ = {}
git_mod.__context__ = {}
log = logging.getLogger(__name__)

WORKTREE_ROOT = '/tmp/salt-tests-tmpdir/main'
WORKTREE_INFO = {
    WORKTREE_ROOT: {
        'HEAD': '119f025073875a938f2456f5ffd7d04e79e5a427',
        'branch': 'refs/heads/master',
        'stale': False,
    },
    '/tmp/salt-tests-tmpdir/worktree1': {
        'HEAD': 'd8d19cf75d7cc3bdc598dc2d472881d26b51a6bf',
        'branch': 'refs/heads/worktree1',
        'stale': False,
    },
    '/tmp/salt-tests-tmpdir/worktree2': {
        'HEAD': '56332ca504aa8b37bb62b54272d52b1d6d832629',
        'branch': 'refs/heads/worktree2',
        'stale': True,
    },
    '/tmp/salt-tests-tmpdir/worktree3': {
        'HEAD': 'e148ea2d521313579f661373fbb93a48a5a6d40d',
        'branch': 'detached',
        'tags': ['v1.1'],
        'stale': False,
    },
    '/tmp/salt-tests-tmpdir/worktree4': {
        'HEAD': '6bbac64d3ad5582b3147088a708952df185db020',
        'branch': 'detached',
        'stale': True,
    },
}


def _git_version():
    git_version = subprocess.Popen(
        ['git', '--version'],
        shell=False,
        close_fds=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE).communicate()[0]
    if not git_version:
        log.error('Git not installed')
        return False
    log.debug('Detected git version ' + git_version)
    return LooseVersion(git_version.split()[-1])


@skipIf(NO_MOCK, NO_MOCK_REASON)
class GitTestCase(TestCase):
    '''
    Test cases for salt.modules.git
    '''
    def test_list_worktrees(self):
        '''
        This tests git.list_worktrees
        '''
        def _build_worktree_output(path):
            '''
            Build 'git worktree list' output for a given path
            '''
            return 'worktree {0}\nHEAD {1}\n{2}\n'.format(
                path,
                WORKTREE_INFO[path]['HEAD'],
                'branch {0}'.format(WORKTREE_INFO[path]['branch'])
                    if WORKTREE_INFO[path]['branch'] != 'detached'
                    else 'detached'
            )

        # Build dict for _cmd_run_side_effect below. Start with the output from
        # 'git worktree list'.
        _cmd_run_values = {
            'git worktree list --porcelain': '\n'.join(
                [_build_worktree_output(x) for x in WORKTREE_INFO]
            ),
            'git --version': 'git version 2.7.0',
        }
        # Add 'git tag --points-at' output for detached HEAD worktrees with
        # tags pointing at HEAD.
        for path in WORKTREE_INFO:
            if WORKTREE_INFO[path]['branch'] != 'detached':
                continue
            key = 'git tag --points-at ' + WORKTREE_INFO[path]['HEAD']
            _cmd_run_values[key] = '\n'.join(
                WORKTREE_INFO[path].get('tags', [])
            )

        def _cmd_run_side_effect(key, **kwargs):
            # Not using dict.get() here because we want to know if
            # _cmd_run_values doesn't account for all uses of cmd.run_all.
            return {'stdout': _cmd_run_values[' '.join(key)],
                    'stderr': '',
                    'retcode': 0,
                    'pid': 12345}

        def _isdir_side_effect(key):
            # os.path.isdir() would return True on a non-stale worktree
            return not WORKTREE_INFO[key].get('stale', False)

        # Build return dict for comparison
        worktree_ret = copy.deepcopy(WORKTREE_INFO)
        for key in worktree_ret:
            ptr = worktree_ret.get(key)
            ptr['detached'] = ptr['branch'] == 'detached'
            ptr['branch'] = None \
                if ptr['detached'] \
                else ptr['branch'].replace('refs/heads/', '', 1)

        cmd_run_mock = MagicMock(side_effect=_cmd_run_side_effect)
        isdir_mock = MagicMock(side_effect=_isdir_side_effect)
        with patch.dict(git_mod.__salt__, {'cmd.run_all': cmd_run_mock}):
            with patch.object(os.path, 'isdir', isdir_mock):
                # Test all=True. Include all return data.
                self.maxDiff = None
                self.assertEqual(
                    git_mod.list_worktrees(
                        WORKTREE_ROOT, all=True, stale=False
                    ),
                    worktree_ret
                )
                # Test all=False and stale=False. Exclude stale worktrees from
                # return data.
                self.assertEqual(
                    git_mod.list_worktrees(
                        WORKTREE_ROOT, all=False, stale=False
                    ),
                    dict([(x, worktree_ret[x]) for x in WORKTREE_INFO
                          if not WORKTREE_INFO[x].get('stale', False)])
                )
                # Test stale=True. Exclude non-stale worktrees from return
                # data.
                self.assertEqual(
                    git_mod.list_worktrees(
                        WORKTREE_ROOT, all=False, stale=True
                    ),
                    dict([(x, worktree_ret[x]) for x in WORKTREE_INFO
                          if WORKTREE_INFO[x].get('stale', False)])
                )


if __name__ == '__main__':
    from integration import run_tests
    run_tests(GitTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import osxdesktop

# Globals
osxdesktop.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class OsxDesktopTestCase(TestCase):
    '''
    Test cases for salt.modules.osxdesktop
    '''
    # 'get_output_volume' function tests: 1

    def test_get_output_volume(self):
        '''
        Test if it get the output volume (range 0 to 100)
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(osxdesktop.__salt__, {'cmd.run': mock}):
            self.assertTrue(osxdesktop.get_output_volume())

    # 'set_output_volume' function tests: 1

    def test_set_output_volume(self):
        '''
        Test if it set the volume of sound (range 0 to 100)
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(osxdesktop.__salt__, {'cmd.run': mock}):
            self.assertTrue(osxdesktop.set_output_volume('my-volume'))

    # 'screensaver' function tests: 1

    def test_screensaver(self):
        '''
        Test if it launch the screensaver
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(osxdesktop.__salt__, {'cmd.run': mock}):
            self.assertTrue(osxdesktop.screensaver())

    # 'lock' function tests: 1

    def test_lock(self):
        '''
        Test if it lock the desktop session
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(osxdesktop.__salt__, {'cmd.run': mock}):
            self.assertTrue(osxdesktop.lock())

    # 'say' function tests: 1

    def test_say(self):
        '''
        Test if it says some words.
        '''
        mock = MagicMock(return_value=True)
        with patch.dict(osxdesktop.__salt__, {'cmd.run': mock}):
            self.assertTrue(osxdesktop.say())


if __name__ == '__main__':
    from integration import run_tests
    run_tests(OsxDesktopTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rupesh Tare <rupesht@saltstack.com>`
'''

# Import Python Libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.modules import qemu_img
import os

# Globals
qemu_img.__salt__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class QemuimgTestCase(TestCase):
    '''
    Test cases for salt.modules.qemu_img
    '''
    def test_make_image(self):
        '''
        Test for create a blank virtual machine image file
        of the specified size in megabytes
        '''
        with patch.object(os.path, 'isabs',
                          MagicMock(side_effect=[False, True, True, True])):
            self.assertEqual(qemu_img.make_image('location', 'size', 'fmt'), '')

            with patch.object(os.path, 'isdir',
                              MagicMock(side_effect=[False, True, True])):
                self.assertEqual(qemu_img.make_image('location', 'size', 'fmt'),
                                 '')

                with patch.dict(qemu_img.__salt__,
                                {'cmd.retcode': MagicMock(side_effect=[False,
                                                                       True])}):
                    self.assertEqual(qemu_img.make_image('location', 'size',
                                                         'fmt'), 'location')

                    self.assertEqual(qemu_img.make_image('location', 'size',
                                                         'fmt'), '')


if __name__ == '__main__':
    from integration import run_tests
    run_tests(QemuimgTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Jayesh Kariya <jayeshk@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import skipIf, TestCase
from salttesting.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../../')

# Import Salt Libs
from salt.states import boto_sqs

boto_sqs.__salt__ = {}
boto_sqs.__opts__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class BotoSqsTestCase(TestCase):
    '''
    Test cases for salt.states.boto_sqs
    '''
    # 'present' function tests: 1

    def test_present(self):
        '''
        Test to ensure the SQS queue exists.
        '''
        name = 'mysqs'
        attributes = {'ReceiveMessageWaitTimeSeconds': 20}

        ret = {'name': name,
               'result': False,
               'changes': {},
               'comment': ''}

        mock = MagicMock(side_effect=[False, False, True, True])
        mock_bool = MagicMock(return_value=False)
        mock_attr = MagicMock(return_value={})
        with patch.dict(boto_sqs.__salt__,
                        {'boto_sqs.exists': mock,
                         'boto_sqs.create': mock_bool,
                         'boto_sqs.get_attributes': mock_attr}):
            with patch.dict(boto_sqs.__opts__, {'test': False}):
                comt = ('Failed to create {0} AWS queue'.format(name))
                ret.update({'comment': comt})
                self.assertDictEqual(boto_sqs.present(name), ret)

            with patch.dict(boto_sqs.__opts__, {'test': True}):
                comt = ('AWS SQS queue {0} is set to be created.'.format(name))
                ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(boto_sqs.present(name), ret)

                comt = ('Attribute(s) ReceiveMessageWaitTimeSeconds'
                        ' to be set on mysqs.')
                ret.update({'comment': comt})
                self.assertDictEqual(boto_sqs.present(name, attributes), ret)

            comt = ('mysqs present. Attributes set.')
            ret.update({'comment': comt, 'result': True})
            self.assertDictEqual(boto_sqs.present(name), ret)

    # 'absent' function tests: 1

    def test_absent(self):
        '''
        Test to ensure the named sqs queue is deleted.
        '''
        name = 'test.example.com.'

        ret = {'name': name,
               'result': True,
               'changes': {},
               'comment': ''}

        mock = MagicMock(side_effect=[False, True])
        with patch.dict(boto_sqs.__salt__,
                        {'boto_sqs.exists': mock}):
            comt = ('{0} does not exist in None.'.format(name))
            ret.update({'comment': comt})
            self.assertDictEqual(boto_sqs.absent(name), ret)

            with patch.dict(boto_sqs.__opts__, {'test': True}):
                comt = ('AWS SQS queue {0} is set to be removed.'.format(name))
                ret.update({'comment': comt, 'result': None})
                self.assertDictEqual(boto_sqs.absent(name), ret)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(BotoSqsTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
Module for managing JBoss AS 7 through the CLI interface.

.. versionadded:: 2015.5.0

In order to run each function, jboss_config dictionary with the following properties must be passed:
 * cli_path: the path to jboss-cli script, for example: '/opt/jboss/jboss-7.0/bin/jboss-cli.sh'
 * controller: the IP address and port of controller, for example: 10.11.12.13:9999
 * cli_user: username to connect to jboss administration console if necessary
 * cli_password: password to connect to jboss administration console if necessary

Example:

.. code-block:: yaml

   jboss_config:
      cli_path: '/opt/jboss/jboss-7.0/bin/jboss-cli.sh'
      controller: 10.11.12.13:9999
      cli_user: 'jbossadm'
      cli_password: 'jbossadm'

'''

# Import python libs
from __future__ import absolute_import
import re
import logging

# Import Salt libs
from salt.utils import dictdiffer

# Import 3rd-party libs
import salt.ext.six as six

log = logging.getLogger(__name__)

__func_alias__ = {
    'reload_': 'reload'
}


def status(jboss_config):
    '''
    Get status of running jboss instance.

    jboss_config
        Configuration dictionary with properties specified above.

    CLI Example:

    .. code-block:: bash

        salt '*' jboss7.status '{"cli_path": "integration.modules.sysmod.SysModuleTest.test_valid_docs", "controller": "10.11.12.13:9999", "cli_user": "jbossadm", "cli_password": "jbossadm"}'

       '''
    log.debug("======================== MODULE FUNCTION: jboss7.status")
    operation = ':read-attribute(name=server-state)'
    return __salt__['jboss7_cli.run_operation'](jboss_config, operation, fail_on_error=False, retries=0)


def stop_server(jboss_config):
    '''
    Stop running jboss instance

    jboss_config
        Configuration dictionary with properties specified above.

    CLI Example:

    .. code-block:: bash

        salt '*' jboss7.stop_server '{"cli_path": "integration.modules.sysmod.SysModuleTest.test_valid_docs", "controller": "10.11.12.13:9999", "cli_user": "jbossadm", "cli_password": "jbossadm"}'

       '''
    log.debug("======================== MODULE FUNCTION: jboss7.stop_server")
    operation = ':shutdown'
    shutdown_result = __salt__['jboss7_cli.run_operation'](jboss_config, operation, fail_on_error=False)
    # JBoss seems to occasionaly close the channel immediately when :shutdown is sent
    if shutdown_result['success'] or (not shutdown_result['success'] and 'Operation failed: Channel closed' in shutdown_result['stdout']):
        return shutdown_result
    else:
        raise Exception('''Cannot handle error, return code={retcode}, stdout='{stdout}', stderr='{stderr}' '''.format(**shutdown_result))


def reload_(jboss_config):
    '''
    Reload running jboss instance

    jboss_config
        Configuration dictionary with properties specified above.

    CLI Example:

    .. code-block:: bash

        salt '*' jboss7.reload '{"cli_path": "integration.modules.sysmod.SysModuleTest.test_valid_docs", "controller": "10.11.12.13:9999", "cli_user": "jbossadm", "cli_password": "jbossadm"}'

       '''
    log.debug("======================== MODULE FUNCTION: jboss7.reload")
    operation = ':reload'
    reload_result = __salt__['jboss7_cli.run_operation'](jboss_config, operation, fail_on_error=False)
    # JBoss seems to occasionaly close the channel immediately when :reload is sent
    if reload_result['success'] or (not reload_result['success'] and
                                         ('Operation failed: Channel closed' in reload_result['stdout'] or
                                          'Communication error: java.util.concurrent.ExecutionException: Operation failed' in reload_result['stdout'])):
        return reload_result
    else:
        raise Exception('''Cannot handle error, return code={retcode}, stdout='{stdout}', stderr='{stderr}' '''.format(**reload_result))


def create_datasource(jboss_config, name, datasource_properties):
    '''
    Create datasource in running jboss instance

    jboss_config
        Configuration dictionary with properties specified above.
    name
        Datasource name
    datasource_properties
        A dictionary of datasource properties to be created:
          - driver-name: mysql
          - connection-url: 'jdbc:mysql://localhost:3306/sampleDatabase'
          - jndi-name: 'java:jboss/datasources/sampleDS'
          - user-name: sampleuser
          - password: secret
          - min-pool-size: 3
          - use-java-context: True

    CLI Example:

    .. code-block:: bash

        salt '*' jboss7.create_datasource '{"cli_path": "integration.modules.sysmod.SysModuleTest.test_valid_docs", "controller": "10.11.12.13:9999", "cli_user": "jbossadm", "cli_password": "jbossadm"}' 'my_datasource' '{"driver-name": "mysql", "connection-url": "jdbc:mysql://localhost:3306/sampleDatabase", "jndi-name": "java:jboss/datasources/sampleDS", "user-name": "sampleuser", "password": "secret", "min-pool-size": 3, "use-java-context": True}'
    '''
    log.debug("======================== MODULE FUNCTION: jboss7.create_datasource, name=%s", name)
    ds_resource_description = __get_datasource_resource_description(jboss_config, name)

    operation = '/subsystem=datasources/data-source="{name}":add({properties})'.format(
                name=name,
                properties=__get_properties_assignment_string(datasource_properties, ds_resource_description)
    )

    return __salt__['jboss7_cli.run_operation'](jboss_config, operation, fail_on_error=False)


def __get_properties_assignment_string(datasource_properties, ds_resource_description):
    assignment_strings = []
    ds_attributes = ds_resource_description['attributes']
    for key, val in six.iteritems(datasource_properties):
        assignment_strings.append(__get_single_assignment_string(key, val, ds_attributes))

    return ','.join(assignment_strings)


def __get_single_assignment_string(key, val, ds_attributes):
    return '{0}={1}'.format(key, __format_value(key, val, ds_attributes))


def __format_value(key, value, ds_attributes):
    type_ = ds_attributes[key]['type']
    if type_ == 'BOOLEAN':
        if value in ('true', 'false'):
            return value
        elif isinstance(value, bool):
            if value:
                return 'true'
            else:
                return 'false'
        else:
            raise Exception("Don't know how to convert {0} to BOOLEAN type".format(value))

    elif type_ == 'INT':
        return str(value)
    elif type_ == 'STRING':
        return '"{0}"'.format(value)
    else:
        raise Exception("Don't know how to format value {0} of type {1}".format(value, type_))


def update_datasource(jboss_config, name, new_properties):
    '''
    Update an existing datasource in running jboss instance.
    If the property doesn't exist if will be created, if it does, it will be updated with the new value

    jboss_config
        Configuration dictionary with properties specified above.
    name
        Datasource name
    new_properties
        A dictionary of datasource properties to be updated. For example:
          - driver-name: mysql
          - connection-url: 'jdbc:mysql://localhost:3306/sampleDatabase'
          - jndi-name: 'java:jboss/datasources/sampleDS'
          - user-name: sampleuser
          - password: secret
          - min-pool-size: 3
          - use-java-context: True

    CLI Example:

    .. code-block:: bash

        salt '*' jboss7.update_datasource '{"cli_path": "integration.modules.sysmod.SysModuleTest.test_valid_docs", "controller": "10.11.12.13:9999", "cli_user": "jbossadm", "cli_password": "jbossadm"}' 'my_datasource' '{"driver-name": "mysql", "connection-url": "jdbc:mysql://localhost:3306/sampleDatabase", "jndi-name": "java:jboss/datasources/sampleDS", "user-name": "sampleuser", "password": "secret", "min-pool-size": 3, "use-java-context": True}'

    '''
    log.debug("======================== MODULE FUNCTION: jboss7.update_datasource, name=%s", name)
    ds_result = __read_datasource(jboss_config, name)
    current_properties = ds_result['result']
    diff = dictdiffer.DictDiffer(new_properties, current_properties)
    changed_properties = diff.changed()

    ret = {
        'success': True,
        'comment': ''
    }
    if len(changed_properties) > 0:
        ds_resource_description = __get_datasource_resource_description(jboss_config, name)
        ds_attributes = ds_resource_description['attributes']
        for key in changed_properties:
            update_result = __update_datasource_property(jboss_config, name, key, new_properties[key], ds_attributes)
            if not update_result['success']:
                ret['result'] = False
                ret['comment'] = ret['comment'] + ('Could not update datasource property {0} with value {1},\n stdout: {2}\n'.format(key, new_properties[key], update_result['stdout']))

    return ret


def __get_datasource_resource_description(jboss_config, name):
    log.debug("======================== MODULE FUNCTION: jboss7.__get_datasource_resource_description, name=%s", name)
    operation = '/subsystem=datasources/data-source="{name}":read-resource-description'.format(name=name)
    operation_result = __salt__['jboss7_cli.run_operation'](jboss_config, operation)
    if operation_result['outcome']:
        return operation_result['result']


def read_datasource(jboss_config, name):
    '''
    Read datasource properties in the running jboss instance.

    jboss_config
        Configuration dictionary with properties specified above.
    name
        Datasource name

    CLI Example:

    .. code-block:: bash

        salt '*' jboss7.read_datasource '{"cli_path": "integration.modules.sysmod.SysModuleTest.test_valid_docs", "controller": "10.11.12.13:9999", "cli_user": "jbossadm", "cli_password": "jbossadm"}'
       '''
    log.debug("======================== MODULE FUNCTION: jboss7.read_datasource, name=%s", name)
    return __read_datasource(jboss_config, name)


def create_simple_binding(jboss_config, binding_name, value):
    '''
    Create a simple jndi binding in the running jboss instance

    jboss_config
        Configuration dictionary with properties specified above.
    binding_name
        Binding name to be created
    value
        Binding value

    CLI Example:

    .. code-block:: bash

        salt '*' jboss7.create_simple_binding \\
                '{"cli_path": "integration.modules.sysmod.SysModuleTest.test_valid_docs", \\
                "controller": "10.11.12.13:9999", "cli_user": "jbossadm", "cli_password": "jbossadm"}' \\
                my_binding_name my_binding_value
    '''
    log.debug("======================== MODULE FUNCTION: jboss7.create_simple_binding, binding_name=%s, value=%s", binding_name, value)
    operation = '/subsystem=naming/binding="{binding_name}":add(binding-type=simple, value="{value}")'.format(
          binding_name=binding_name,
          value=__escape_binding_value(value)
    )
    return __salt__['jboss7_cli.run_operation'](jboss_config, operation)


def update_simple_binding(jboss_config, binding_name, value):
    '''
    Update the simple jndi binding in the running jboss instance

    jboss_config
        Configuration dictionary with properties specified above.
    binding_name
        Binding name to be updated
    value
        New binding value

    CLI Example:

    .. code-block:: bash

        salt '*' jboss7.update_simple_binding '{"cli_path": "integration.modules.sysmod.SysModuleTest.test_valid_docs", "controller": "10.11.12.13:9999", "cli_user": "jbossadm", "cli_password": "jbossadm"}' my_binding_name my_binding_value
       '''
    log.debug("======================== MODULE FUNCTION: jboss7.update_simple_binding, binding_name=%s, value=%s", binding_name, value)
    operation = '/subsystem=naming/binding="{binding_name}":write-attribute(name=value, value="{value}")'.format(
        binding_name=binding_name,
        value=__escape_binding_value(value)
    )
    return __salt__['jboss7_cli.run_operation'](jboss_config, operation)


def read_simple_binding(jboss_config, binding_name):
    '''
    Read jndi binding in the running jboss instance

    jboss_config
        Configuration dictionary with properties specified above.
    binding_name
        Binding name to be created

    CLI Example:

        .. code-block:: bash

        salt '*' jboss7.read_simple_binding '{"cli_path": "integration.modules.sysmod.SysModuleTest.test_valid_docs", "controller": "10.11.12.13:9999", "cli_user": "jbossadm", "cli_password": "jbossadm"}' my_binding_name
       '''
    log.debug("======================== MODULE FUNCTION: jboss7.read_simple_binding, %s", binding_name)
    return __read_simple_binding(jboss_config, binding_name)


def __read_simple_binding(jboss_config, binding_name):
    operation = '/subsystem=naming/binding="{binding_name}":read-resource'.format(binding_name=binding_name)
    return __salt__['jboss7_cli.run_operation'](jboss_config, operation)


def __update_datasource_property(jboss_config, datasource_name, name, value, ds_attributes):
    log.debug("======================== MODULE FUNCTION: jboss7.__update_datasource_property, datasource_name=%s, name=%s, value=%s", datasource_name, name, value)
    operation = '/subsystem=datasources/data-source="{datasource_name}":write-attribute(name="{name}",value={value})'.format(
                  datasource_name=datasource_name,
                  name=name,
                  value=__format_value(name, value, ds_attributes)
            )
    return __salt__['jboss7_cli.run_operation'](jboss_config, operation, fail_on_error=False)


def __read_datasource(jboss_config, name):
    operation = '/subsystem=datasources/data-source="{name}":read-resource'.format(name=name)
    operation_result = __salt__['jboss7_cli.run_operation'](jboss_config, operation)

    return operation_result


def __escape_binding_value(binding_name):
    result = binding_name.replace('\\', '\\\\\\\\')  # replace \ -> \\\\

    return result


def remove_datasource(jboss_config, name):
    '''
    Remove an existing datasource from the running jboss instance.

    jboss_config
        Configuration dictionary with properties specified above.
    name
        Datasource name

    CLI Example:

    .. code-block:: bash

        salt '*' jboss7.remove_datasource '{"cli_path": "integration.modules.sysmod.SysModuleTest.test_valid_docs", "controller": "10.11.12.13:9999", "cli_user": "jbossadm", "cli_password": "jbossadm"}' my_datasource_name
       '''
    log.debug("======================== MODULE FUNCTION: jboss7.remove_datasource, name=%s", name)
    operation = '/subsystem=datasources/data-source={name}:remove'.format(name=name)
    return __salt__['jboss7_cli.run_operation'](jboss_config, operation, fail_on_error=False)


def deploy(jboss_config, source_file):
    '''
    Deploy the application on the jboss instance from the local file system where minion is running.

    jboss_config
        Configuration dictionary with properties specified above.
    source_file
        Source file to deploy from

    CLI Example:

    .. code-block:: bash

        salt '*' jboss7.deploy '{"cli_path": "integration.modules.sysmod.SysModuleTest.test_valid_docs", "controller": "10.11.12.13:9999", "cli_user": "jbossadm", "cli_password": "jbossadm"}' /opt/deploy_files/my_deploy
       '''
    log.debug("======================== MODULE FUNCTION: jboss7.deploy, source_file=%s", source_file)
    command = 'deploy {source_file} --force '.format(source_file=source_file)
    return __salt__['jboss7_cli.run_command'](jboss_config, command, fail_on_error=False)


def list_deployments(jboss_config):
    '''
    List all deployments on the jboss instance

    jboss_config
        Configuration dictionary with properties specified above.

     CLI Example:

     .. code-block:: bash

         salt '*' jboss7.list_deployments '{"cli_path": "integration.modules.sysmod.SysModuleTest.test_valid_docs", "controller": "10.11.12.13:9999", "cli_user": "jbossadm", "cli_password": "jbossadm"}'

       '''
    log.debug("======================== MODULE FUNCTION: jboss7.list_deployments")
    command_result = __salt__['jboss7_cli.run_command'](jboss_config, 'deploy')
    deployments = []
    if len(command_result['stdout']) > 0:
        deployments = re.split('\\s*', command_result['stdout'])
    log.debug('deployments=%s', str(deployments))
    return deployments


def undeploy(jboss_config, deployment):
    '''
    Undeploy the application from jboss instance

    jboss_config
        Configuration dictionary with properties specified above.
    deployment
        Deployment name to undeploy

    CLI Example:

    .. code-block:: bash

        salt '*' jboss7.undeploy '{"cli_path": "integration.modules.sysmod.SysModuleTest.test_valid_docs", "controller": "10.11.12.13:9999", "cli_user": "jbossadm", "cli_password": "jbossadm"}' my_deployment
       '''
    log.debug("======================== MODULE FUNCTION: jboss7.undeploy, deployment=%s", deployment)
    command = 'undeploy {deployment} '.format(deployment=deployment)
    return __salt__['jboss7_cli.run_command'](jboss_config, command)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Mike Place <mp@saltstack.com>`
'''

# Import python libs
from __future__ import absolute_import
import os

# Import Salt Testing libs
from salttesting import TestCase, skipIf
from salttesting.helpers import ensure_in_syspath
from salttesting.mock import NO_MOCK, NO_MOCK_REASON, patch

# Import salt libs
from salt import minion
from salt.utils import event
from salt.exceptions import SaltSystemExit
import salt.syspaths

ensure_in_syspath('../')

__opts__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class MinionTestCase(TestCase):
    def test_invalid_master_address(self):
        with patch.dict(__opts__, {'ipv6': False, 'master': float('127.0'), 'master_port': '4555', 'retry_dns': False}):
            self.assertRaises(SaltSystemExit, minion.resolve_dns, __opts__)

    @skipIf(os.geteuid() != 0, 'You must be logged in as root to run this test')
    def test_sock_path_len(self):
        '''
        This tests whether or not a larger hash causes the sock path to exceed
        the system's max sock path length. See the below link for more
        information.

        https://github.com/saltstack/salt/issues/12172#issuecomment-43903643
        '''
        opts = {
            'id': 'salt-testing',
            'hash_type': 'sha512',
            'sock_dir': os.path.join(salt.syspaths.SOCK_DIR, 'minion'),
            'extension_modules': ''
        }
        with patch.dict(__opts__, opts):
            try:
                event_publisher = event.AsyncEventPublisher(__opts__, lambda x: True)
                result = True
            except SaltSystemExit:
                result = False
        self.assertTrue(result)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(MinionTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Rupesh Tare <rupesht@saltstack.com>`
'''
# Import Python libs
from __future__ import absolute_import

# Import Salt Testing Libs
from salttesting import TestCase, skipIf
from salttesting.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
from salt.modules import cassandra


cassandra.__grains__ = {}
cassandra.__salt__ = {}
cassandra.__context__ = {}
cassandra.__opts__ = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class CassandraTestCase(TestCase):
    '''
    Test cases for salt.modules.cassandra
    '''
    def test_compactionstats(self):
        '''
        Test for Return compactionstats info
        '''
        mock = MagicMock(return_value='A')
        with patch.object(cassandra, '_nodetool', mock):
            self.assertEqual(cassandra.compactionstats(), 'A')

    def test_version(self):
        '''
        Test for Return the cassandra version
        '''
        mock = MagicMock(return_value='A')
        with patch.object(cassandra, '_nodetool', mock):
            self.assertEqual(cassandra.version(), 'A')

    def test_netstats(self):
        '''
        Test for Return netstats info
        '''
        mock = MagicMock(return_value='A')
        with patch.object(cassandra, '_nodetool', mock):
            self.assertEqual(cassandra.netstats(), 'A')

    def test_tpstats(self):
        '''
        Test for Return tpstats info
        '''
        mock = MagicMock(return_value='A')
        with patch.object(cassandra, '_nodetool', mock):
            self.assertEqual(cassandra.tpstats(), 'A')

    def test_info(self):
        '''
        Test for Return cassandra node info
        '''
        mock = MagicMock(return_value='A')
        with patch.object(cassandra, '_nodetool', mock):
            self.assertEqual(cassandra.info(), 'A')

    def test_ring(self):
        '''
        Test for Return ring info
        '''
        mock = MagicMock(return_value='A')
        with patch.object(cassandra, '_nodetool', mock):
            self.assertEqual(cassandra.ring(), 'A')

    def test_keyspaces(self):
        '''
        Test for Return existing keyspaces
        '''
        mock_keyspaces = ['A', 'B', 'C', 'D']

        class MockSystemManager(object):
            def list_keyspaces(self):
                return mock_keyspaces

        mock_sys_mgr = MagicMock(return_value=MockSystemManager())

        with patch.object(cassandra, '_sys_mgr', mock_sys_mgr):
            self.assertEqual(cassandra.keyspaces(), mock_keyspaces)

    def test_column_families(self):
        '''
        Test for Return existing column families for all keyspaces
        '''
        mock_keyspaces = ['A', 'B']

        class MockSystemManager(object):
            def list_keyspaces(self):
                return mock_keyspaces

            def get_keyspace_column_families(self, keyspace):
                if keyspace == 'A':
                    return {'a': 'saltines', 'b': 'biscuits'}
                if keyspace == 'B':
                    return {'c': 'cheese', 'd': 'crackers'}

        mock_sys_mgr = MagicMock(return_value=MockSystemManager())

        with patch.object(cassandra, '_sys_mgr', mock_sys_mgr):
            self.assertEqual(cassandra.column_families('A'),
                             ['a', 'b'])
            self.assertEqual(cassandra.column_families('Z'),
                             None)
            self.assertEqual(cassandra.column_families(),
                             {'A': ['a', 'b'], 'B': ['c', 'd']})

    def test_column_family_definition(self):
        '''
        Test for Return a dictionary of column family definitions for the given
        keyspace/column_family
        '''
        class MockSystemManager(object):
            def get_keyspace_column_families(self, keyspace):
                if keyspace == 'A':
                    return {'a': object, 'b': object}
                if keyspace == 'B':
                    raise Exception

        mock_sys_mgr = MagicMock(return_value=MockSystemManager())

        with patch.object(cassandra, '_sys_mgr', mock_sys_mgr):
            self.assertEqual(cassandra.column_family_definition('A', 'a'), vars(object))
            self.assertEqual(cassandra.column_family_definition('B', 'a'), None)


if __name__ == '__main__':
    from integration import run_tests
    run_tests(CassandraTestCase, needs_daemon=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Nicole Thomas <nicole@saltstack.com>`
    :codeauthor: :email:`Tomas Sirny <tsirny@gmail.com>`
'''

# Import Python Libs
from __future__ import absolute_import
import os
import random
import string

# Import Salt Libs
import integration
from salt.config import cloud_providers_config

# Import Salt Testing Libs
from salttesting.helpers import ensure_in_syspath, expensiveTest

ensure_in_syspath('../../../')

# Import Third-Party Libs
from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin


def _random_name(size=6):
    '''
    Generates a radom cloud instance name
    '''
    return 'cloud-test-' + ''.join(
        random.choice(string.ascii_lowercase + string.digits)
        for x in range(size)
    )


class GCETest(integration.ShellCase):
    '''
    Integration tests for the GCE cloud provider in Salt-Cloud
    '''

    @expensiveTest
    def setUp(self):
        '''
        Sets up the test requirements
        '''
        super(GCETest, self).setUp()

        # check if appropriate cloud provider and profile files are present
        profile_str = 'gce-config:'
        provider = 'gce'
        providers = self.run_cloud('--list-providers')
        # Create the cloud instance name to be used throughout the tests
        self.INSTANCE_NAME = _random_name()

        if profile_str not in providers:
            self.skipTest(
                'Configuration file for {0} was not found. Check {0}.conf files '
                'in tests/integration/files/conf/cloud.*.d/ to run these tests.'
                .format(provider)
            )

        # check if project, service_account_email_address, service_account_private_key
        # and provider are present
        path = os.path.join(integration.FILES,
                            'conf',
                            'cloud.providers.d',
                            provider + '.conf')
        config = cloud_providers_config(path)

        project = config['gce-config']['gce']['project']
        service_account_email_address = config['gce-config']['gce']['service_account_email_address']
        service_account_private_key = config['gce-config']['gce']['service_account_private_key']

        conf_items = [project, service_account_email_address, service_account_private_key]
        missing_conf_item = []

        for item in conf_items:
            if item == '':
                missing_conf_item.append(item)

        if missing_conf_item:
            self.skipTest(
                'An project, service_account_email_address, service_account_private_key must '
                'be provided to run these tests. One or more of these elements is '
                'missing. Check tests/integration/files/conf/cloud.providers.d/{0}.conf'
                .format(provider)
            )

    def test_instance(self):
        '''
        Tests creating and deleting an instance on GCE
        '''

        # create the instance
        instance = self.run_cloud('-p gce-test {0}'.format(self.INSTANCE_NAME))
        ret_str = '{0}:'.format(self.INSTANCE_NAME)

        # check if instance returned with salt installed
        try:
            self.assertIn(ret_str, instance)
        except AssertionError:
            self.run_cloud('-d {0} --assume-yes'.format(self.INSTANCE_NAME))
            raise

        # delete the instance
        delete = self.run_cloud('-d {0} --assume-yes'.format(self.INSTANCE_NAME))
        # example response: ['gce-config:', '----------', '    gce:', '----------', 'cloud-test-dq4e6c:', 'True', '']
        delete_str = ''.join(delete)

        # check if deletion was performed appropriately
        try:
            self.assertIn(self.INSTANCE_NAME, delete_str)
            self.assertIn('True', delete_str)
        except AssertionError:
            raise

    def test_instance_extra(self):
        '''
        Tests creating and deleting an instance on GCE
        '''

        # create the instance
        instance = self.run_cloud('-p gce-test-extra {0}'.format(self.INSTANCE_NAME))
        ret_str = '{0}:'.format(self.INSTANCE_NAME)

        # check if instance returned with salt installed
        try:
            self.assertIn(ret_str, instance)
        except AssertionError:
            self.run_cloud('-d {0} --assume-yes'.format(self.INSTANCE_NAME))
            raise

        # delete the instance
        delete = self.run_cloud('-d {0} --assume-yes'.format(self.INSTANCE_NAME))
        # example response: ['gce-config:', '----------', '    gce:', '----------', 'cloud-test-dq4e6c:', 'True', '']
        delete_str = ''.join(delete)

        # check if deletion was performed appropriately
        try:
            self.assertIn(self.INSTANCE_NAME, delete_str)
            self.assertIn('True', delete_str)
        except AssertionError:
            raise

    def tearDown(self):
        '''
        Clean up after tests
        '''
        # salt-cloud -a show_instance myinstance
        query = self.run_cloud('--query')
        ret_str = '        {0}:'.format(self.INSTANCE_NAME)

        # if test instance is still present, delete it
        if ret_str in query:
            self.run_cloud('-d {0} --assume-yes'.format(self.INSTANCE_NAME))


if __name__ == '__main__':
    from integration import run_tests
    run_tests(GCETest)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import

# Import Salt Testing libs
from salttesting import skipIf, TestCase
from salttesting.helpers import ensure_in_syspath

ensure_in_syspath('../..')

# Import Salt libs
import salt.state
from salt.config import minion_config
from salt.template import compile_template_str
from salt.serializers import yamlex

basic_template = '''#!yamlex
foo: bar
'''

complex_template = '''#!yamlex
placeholder: {foo: !aggregate {foo: 42}}
placeholder: {foo: !aggregate {bar: null}}
placeholder: {foo: !aggregate {baz: inga}}
'''

SKIP_MESSAGE = '%s is unavailable, do prerequisites have been met?'


class RendererMixin(object):
    def render(self, template, opts=None):
        _config = minion_config(None)
        _config['file_client'] = 'local'
        if opts:
            _config.update(opts)
        _state = salt.state.State(_config)
        return compile_template_str(template,
                                    _state.rend,
                                    _state.opts['renderer'])


class RendererTests(TestCase, RendererMixin):
    @skipIf(not yamlex.available, SKIP_MESSAGE % 'yamlex')
    def test_basic(self):
        sls_obj = self.render(basic_template)
        assert sls_obj == {'foo': 'bar'}, sls_obj

    @skipIf(not yamlex.available, SKIP_MESSAGE % 'yamlex')
    def test_complex(self):

        sls_obj = self.render(complex_template)
        assert sls_obj == {
            'placeholder': {
                'foo': {
                    'foo': 42,
                    'bar': None,
                    'baz': 'inga'
                }
            }
        }, sls_obj

if __name__ == '__main__':
    from integration import run_tests
    run_tests(RendererTests, needs_daemon=False)



# encoding: utf-8
'''
The main entry point for salt-api
'''
from __future__ import absolute_import
# Import python libs
import logging

# Import salt-api libs
import salt.loader
import salt.utils.process

logger = logging.getLogger(__name__)


class NetapiClient(object):
    '''
    Start each netapi module that is configured to run
    '''
    def __init__(self, opts):
        self.opts = opts
        self.process_manager = salt.utils.process.ProcessManager()
        self.netapi = salt.loader.netapi(self.opts)

    def run(self):
        '''
        Load and start all available api modules
        '''
        for fun in self.netapi:
            if fun.endswith('.start'):
                logger.info('Starting {0} netapi module'.format(fun))
                self.process_manager.add_process(self.netapi[fun])

        self.process_manager.run()



# -*- coding: utf-8 -*-
'''
This is a simple proxy-minion designed to connect to and communicate with
the bottle-based web service contained in https://github.com/saltstack/salt-contrib/tree/master/proxyminion_rest_example
'''
from __future__ import absolute_import

# Import python libs
import logging
import salt.utils.http

HAS_REST_EXAMPLE = True

# This must be present or the Salt loader won't load this module
__proxyenabled__ = ['rest_sample']


# Variables are scoped to this module so we can have persistent data
# across calls to fns in here.
GRAINS_CACHE = {}
DETAILS = {}

# Want logging!
log = logging.getLogger(__file__)


# This does nothing, it's here just as an example and to provide a log
# entry when the module is loaded.
def __virtual__():
    '''
    Only return if all the modules are available
    '''
    log.debug('rest_sample proxy __virtual__() called...')
    return True

# Every proxy module needs an 'init', though you can
# just put a 'pass' here if it doesn't need to do anything.


def init(opts):
    log.debug('rest_sample proxy init() called...')

    # Save the REST URL
    DETAILS['url'] = opts['proxy']['url']

    # Make sure the REST URL ends with a '/'
    if not DETAILS['url'].endswith('/'):
        DETAILS['url'] += '/'


def id(opts):
    '''
    Return a unique ID for this proxy minion.  This ID MUST NOT CHANGE.
    If it changes while the proxy is running the salt-master will get
    really confused and may stop talking to this minion
    '''
    r = salt.utils.http.query(opts['proxy']['url']+'id', decode_type='json', decode=True)
    return r['dict']['id'].encode('ascii', 'ignore')


def grains():
    '''
    Get the grains from the proxied device
    '''
    if not GRAINS_CACHE:
        r = salt.utils.http.query(DETAILS['url']+'info', decode_type='json', decode=True)
        GRAINS_CACHE = r['dict']
    return GRAINS_CACHE


def grains_refresh():
    '''
    Refresh the grains from the proxied device
    '''
    GRAINS_CACHE = {}
    return grains()


def service_start(name):
    '''
    Start a "service" on the REST server
    '''
    r = salt.utils.http.query(DETAILS['url']+'service/start/'+name, decode_type='json', decode=True)
    return r['dict']


def service_stop(name):
    '''
    Stop a "service" on the REST server
    '''
    r = salt.utils.http.query(DETAILS['url']+'service/stop/'+name, decode_type='json', decode=True)
    return r['dict']


def service_restart(name):
    '''
    Restart a "service" on the REST server
    '''
    r = salt.utils.http.query(DETAILS['url']+'service/restart/'+name, decode_type='json', decode=True)
    return r['dict']


def service_list():
    '''
    List "services" on the REST server
    '''
    r = salt.utils.http.query(DETAILS['url']+'service/list', decode_type='json', decode=True)
    return r['dict']


def service_status(name):
    '''
    Check if a service is running on the REST server
    '''
    r = salt.utils.http.query(DETAILS['url']+'service/status/'+name, decode_type='json', decode=True)
    return r['dict']


def package_list():
    '''
    List "packages" installed on the REST server
    '''
    r = salt.utils.http.query(DETAILS['url']+'package/list', decode_type='json', decode=True)
    return r['dict']


def package_install(name, **kwargs):
    '''
    Install a "package" on the REST server
    '''
    cmd = DETAILS['url']+'package/install/'+name
    if 'version' in kwargs:
        cmd += '/'+kwargs['version']
    else:
        cmd += '/1.0'
    r = salt.utils.http.query(cmd, decode_type='json', decode=True)
    return r['dict']


def package_remove(name):

    '''
    Remove a "package" on the REST server
    '''
    r = salt.utils.http.query(DETAILS['url']+'package/remove/'+name, decode_type='json', decode=True)
    return r['dict']


def package_status(name):
    '''
    Check the installation status of a package on the REST server
    '''
    r = salt.utils.http.query(DETAILS['url']+'package/status/'+name, decode_type='json', decode=True)
    return r['dict']


def ping():
    '''
    Is the REST server up?
    '''
    r = salt.utils.http.query(DETAILS['url']+'ping', decode_type='json', decode=True)
    try:
        return r['dict'].get('ret', False)
    except Exception:
        return False


def shutdown(opts):
    '''
    For this proxy shutdown is a no-op
    '''
    log.debug('rest_sample proxy shutdown() called...')
